# -*- coding: utf-8 -*-
'''
These only test the provider selection and verification logic, they do not init
any remotes.
'''

# Import python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import MagicMock, patch, NO_MOCK, NO_MOCK_REASON

# Import salt libs
import salt.utils.gitfs
from salt.exceptions import FileserverConfigError

# GLOBALS
OPTS = {'cachedir': '/tmp/gitfs-test-cache'}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class TestGitFSProvider(TestCase):

    def test_provider_case_insensitive(self):
        '''
        Ensure that both lowercase and non-lowercase values are supported
        '''
        provider = 'GitPython'
        for role_name, role_class in (
                ('gitfs', salt.utils.gitfs.GitFS),
                ('git_pillar', salt.utils.gitfs.GitPillar),
                ('winrepo', salt.utils.gitfs.WinRepo)):

            key = '{0}_provider'.format(role_name)
            with patch.object(role_class, 'verify_gitpython',
                              MagicMock(return_value=True)):
                with patch.object(role_class, 'verify_pygit2',
                                  MagicMock(return_value=False)):
                    args = [OPTS, {}]
                    kwargs = {'init_remotes': False}
                    if role_name == 'winrepo':
                        kwargs['cache_root'] = '/tmp/winrepo-dir'
                    with patch.dict(OPTS, {key: provider}):
                        # Try to create an instance with uppercase letters in
                        # provider name. If it fails then a
                        # FileserverConfigError will be raised, so no assert is
                        # necessary.
                        role_class(*args, **kwargs)
                    # Now try to instantiate an instance with all lowercase
                    # letters. Again, no need for an assert here.
                    role_class(*args, **kwargs)

    def test_valid_provider(self):
        '''
        Ensure that an invalid provider is not accepted, raising a
        FileserverConfigError.
        '''
        def _get_mock(verify, provider):
            '''
            Return a MagicMock with the desired return value
            '''
            return MagicMock(return_value=verify.endswith(provider))

        for role_name, role_class in (
                ('gitfs', salt.utils.gitfs.GitFS),
                ('git_pillar', salt.utils.gitfs.GitPillar),
                ('winrepo', salt.utils.gitfs.WinRepo)):
            key = '{0}_provider'.format(role_name)
            for provider in salt.utils.gitfs.GIT_PROVIDERS:
                verify = 'verify_gitpython'
                mock1 = _get_mock(verify, provider)
                with patch.object(role_class, verify, mock1):
                    verify = 'verify_pygit2'
                    mock2 = _get_mock(verify, provider)
                    with patch.object(role_class, verify, mock2):
                        args = [OPTS, {}]
                        kwargs = {'init_remotes': False}
                        if role_name == 'winrepo':
                            kwargs['cache_root'] = '/tmp/winrepo-dir'

                        with patch.dict(OPTS, {key: provider}):
                            role_class(*args, **kwargs)

                        with patch.dict(OPTS, {key: 'foo'}):
                            # Set the provider name to a known invalid provider
                            # and make sure it raises an exception.
                            self.assertRaises(
                                FileserverConfigError,
                                role_class,
                                *args,
                                **kwargs)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import shutil
import stat

# Import Salt Testing libs
from tests.support.case import ShellCase
from tests.support.paths import TMP, INTEGRATION_TEST_DIR

# Import 3rd-party libs

# Import Salt libs
import salt.utils.files
import salt.utils.stringutils

# all read, only owner write
autosign_file_permissions = stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH | stat.S_IWUSR
autosign_file_path = os.path.join(TMP, 'rootdir', 'autosign_file')


class AutosignGrainsTest(ShellCase):
    '''
    Test autosigning minions based on grain values.
    '''

    def setUp(self):
        shutil.copyfile(
            os.path.join(INTEGRATION_TEST_DIR, 'files', 'autosign_grains', 'autosign_file'),
            autosign_file_path
        )
        os.chmod(autosign_file_path, autosign_file_permissions)

        self.run_key('-d minion -y')
        self.run_call('test.ping -l quiet')  # get minon to try to authenticate itself again

        if 'minion' in self.run_key('-l acc'):
            self.tearDown()
            self.skipTest('Could not deauthorize minion')
        if 'minion' not in self.run_key('-l un'):
            self.tearDown()
            self.skipTest('minion did not try to reauthenticate itself')

        self.autosign_grains_dir = os.path.join(self.master_opts['autosign_grains_dir'])
        if not os.path.isdir(self.autosign_grains_dir):
            os.makedirs(self.autosign_grains_dir)

    def tearDown(self):
        shutil.copyfile(
            os.path.join(INTEGRATION_TEST_DIR, 'files', 'autosign_file'),
            autosign_file_path
        )
        os.chmod(autosign_file_path, autosign_file_permissions)

        self.run_call('test.ping -l quiet')  # get minon to authenticate itself again

        if os.path.isdir(self.autosign_grains_dir):
            shutil.rmtree(self.autosign_grains_dir)

    def test_autosign_grains_accept(self):
        grain_file_path = os.path.join(self.autosign_grains_dir, 'test_grain')
        with salt.utils.files.fopen(grain_file_path, 'w') as f:
            f.write(salt.utils.stringutils.to_str('#invalid_value\ncheese'))
        os.chmod(grain_file_path, autosign_file_permissions)

        self.run_call('test.ping -l quiet')  # get minon to try to authenticate itself again
        self.assertIn('minion', self.run_key('-l acc'))

    def test_autosign_grains_fail(self):
        grain_file_path = os.path.join(self.autosign_grains_dir, 'test_grain')
        with salt.utils.files.fopen(grain_file_path, 'w') as f:
            f.write(salt.utils.stringutils.to_str('#cheese\ninvalid_value'))
        os.chmod(grain_file_path, autosign_file_permissions)

        self.run_call('test.ping -l quiet')  # get minon to try to authenticate itself again
        self.assertNotIn('minion', self.run_key('-l acc'))
        self.assertIn('minion', self.run_key('-l un'))



# -*- coding: utf-8 -*-
'''
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.case import ShellCase

# Import Salt libs
import salt.utils.platform


class BatchTest(ShellCase):
    '''
    Integration tests for the salt.cli.batch module
    '''
    if salt.utils.platform.is_windows():
        run_timeout = 90
    else:
        run_timeout = 30

    def test_batch_run(self):
        '''
        Tests executing a simple batch command to help catch regressions
        '''
        ret = 'Executing run on [{0}]'.format(repr('sub_minion'))
        cmd = self.run_salt(
            '"*minion" test.echo "batch testing" -b 50%',
            timeout=self.run_timeout,
        )
        self.assertIn(ret, cmd)

    def test_batch_run_number(self):
        '''
        Tests executing a simple batch command using a number division instead of
        a percentage with full batch CLI call.
        '''
        ret = "Executing run on [{0}, {1}]".format(repr('minion'), repr('sub_minion'))
        cmd = self.run_salt(
            '"*minion" test.ping --batch-size 2',
            timeout=self.run_timeout,
        )
        self.assertIn(ret, cmd)

    def test_batch_run_grains_targeting(self):
        '''
        Tests executing a batch command using a percentage divisor as well as grains
        targeting.
        '''
        os_grain = ''
        sub_min_ret = "Executing run on [{0}]".format(repr('sub_minion'))
        min_ret = "Executing run on [{0}]".format(repr('minion'))

        for item in self.run_salt('minion grains.get os'):
            if item != 'minion:':
                os_grain = item

        os_grain = os_grain.strip()
        cmd = self.run_salt(
            '-C "G@os:{0} and not localhost" -b 25% test.ping'.format(os_grain),
            timeout=self.run_timeout,
        )
        self.assertIn(sub_min_ret, cmd)
        self.assertIn(min_ret, cmd)

    def test_batch_exit_code(self):
        '''
        Test that a failed state returns a non-zero exit code in batch mode
        '''
        cmd = self.run_salt(
            ' "*" state.single test.fail_without_changes name=test_me -b 25%',
            with_retcode=True,
            timeout=self.run_timeout,
        )
        self.assertEqual(cmd[-1], 2)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ModuleCase

# Import 3rd-party libs
from salt.ext import six


class StdTest(ModuleCase):
    '''
    Test standard client calls
    '''
    def test_cli(self):
        '''
        Test cli function
        '''
        cmd_iter = self.client.cmd_cli(
                'minion',
                'test.arg',
                ['foo', 'bar', 'baz'],
                kwarg={'qux': 'quux'}
                )
        for ret in cmd_iter:
            data = ret['minion']['ret']
            self.assertEqual(data['args'], ['foo', 'bar', 'baz'])
            self.assertEqual(data['kwargs']['qux'], 'quux')

    def test_iter(self):
        '''
        test cmd_iter
        '''
        cmd_iter = self.client.cmd_iter(
                'minion',
                'test.arg',
                ['foo', 'bar', 'baz'],
                kwarg={'qux': 'quux'}
                )
        for ret in cmd_iter:
            data = ret['minion']['ret']
            self.assertEqual(data['args'], ['foo', 'bar', 'baz'])
            self.assertEqual(data['kwargs']['qux'], 'quux')

    def test_iter_no_block(self):
        '''
        test cmd_iter_no_block
        '''
        cmd_iter = self.client.cmd_iter_no_block(
                'minion',
                'test.arg',
                ['foo', 'bar', 'baz'],
                kwarg={'qux': 'quux'}
                )
        for ret in cmd_iter:
            if ret is None:
                continue
            data = ret['minion']['ret']
            self.assertEqual(data['args'], ['foo', 'bar', 'baz'])
            self.assertEqual(data['kwargs']['qux'], 'quux')

    def test_full_returns(self):
        '''
        test cmd_iter
        '''
        ret = self.client.cmd_full_return(
                'minion',
                'test.arg',
                ['foo', 'bar', 'baz'],
                kwarg={'qux': 'quux'}
                )
        data = ret['minion']['ret']
        self.assertEqual(data['args'], ['foo', 'bar', 'baz'])
        self.assertEqual(data['kwargs']['qux'], 'quux')

    def test_kwarg_type(self):
        '''
        Test that kwargs end up on the client as the same type
        '''
        terrible_yaml_string = 'foo: ""\n# \''
        ret = self.client.cmd_full_return(
                'minion',
                'test.arg_type',
                ['a', 1],
                kwarg={'outer': {'a': terrible_yaml_string},
                       'inner': 'value'}
                )
        data = ret['minion']['ret']
        self.assertIn(six.text_type.__name__, data['args'][0])
        self.assertIn('int', data['args'][1])
        self.assertIn('dict', data['kwargs']['outer'])
        self.assertIn(six.text_type.__name__, data['kwargs']['inner'])

    def test_full_return_kwarg(self):
        ret = self.client.cmd('minion', 'test.ping', full_return=True)
        for mid, data in ret.items():
            self.assertIn('retcode', data)

    def test_cmd_arg_kwarg_parsing(self):
        ret = self.client.cmd('minion', 'test.arg_clean',
            arg=[
                'foo',
                'bar=off',
                'baz={qux: 123}'
            ],
            kwarg={
                'quux': 'Quux',
            })

        self.assertEqual(ret['minion'], {
            'args': ['foo'],
            'kwargs': {
                'bar': False,
                'baz': {
                    'qux': 123,
                },
                'quux': 'Quux',
            },
        })



# -*- coding: utf-8 -*-
'''
Unit Tests for functions located in salt.utils.state.py.
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import copy
import textwrap

# Import Salt libs
from salt.ext import six
import salt.utils.odict
import salt.utils.state

# Import Salt Testing libs
from tests.support.unit import TestCase


class StateUtilTestCase(TestCase):
    '''
    Test case for state util.
    '''
    def test_check_result(self):
        self.assertFalse(salt.utils.state.check_result(None),
                         'Failed to handle None as an invalid data type.')
        self.assertFalse(salt.utils.state.check_result([]),
                         'Failed to handle an invalid data type.')
        self.assertFalse(salt.utils.state.check_result({}),
                         'Failed to handle an empty dictionary.')
        self.assertFalse(salt.utils.state.check_result({'host1': []}),
                         'Failed to handle an invalid host data structure.')
        test_valid_state = {'host1': {'test_state': {'result': 'We have liftoff!'}}}
        self.assertTrue(salt.utils.state.check_result(test_valid_state))
        test_valid_false_states = {
            'test1': salt.utils.odict.OrderedDict([
                ('host1',
                 salt.utils.odict.OrderedDict([
                     ('test_state0', {'result':  True}),
                     ('test_state', {'result': False}),
                 ])),
            ]),
            'test2': salt.utils.odict.OrderedDict([
                ('host1',
                 salt.utils.odict.OrderedDict([
                     ('test_state0', {'result':  True}),
                     ('test_state', {'result': True}),
                 ])),
                ('host2',
                 salt.utils.odict.OrderedDict([
                     ('test_state0', {'result':  True}),
                     ('test_state', {'result': False}),
                 ])),
            ]),
            'test3': ['a'],
            'test4': salt.utils.odict.OrderedDict([
                ('asup', salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('test_state0', {'result':  True}),
                         ('test_state', {'result': True}),
                     ])),
                    ('host2',
                     salt.utils.odict.OrderedDict([
                         ('test_state0', {'result':  True}),
                         ('test_state', {'result': False}),
                     ]))
                ]))
            ]),
            'test5': salt.utils.odict.OrderedDict([
                ('asup', salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('test_state0', {'result':  True}),
                         ('test_state', {'result': True}),
                     ])),
                    ('host2', salt.utils.odict.OrderedDict([]))
                ]))
            ])
        }
        for test, data in six.iteritems(test_valid_false_states):
            self.assertFalse(
                salt.utils.state.check_result(data),
                msg='{0} failed'.format(test))
        test_valid_true_states = {
            'test1': salt.utils.odict.OrderedDict([
                ('host1',
                 salt.utils.odict.OrderedDict([
                     ('test_state0', {'result':  True}),
                     ('test_state', {'result': True}),
                 ])),
            ]),
            'test3': salt.utils.odict.OrderedDict([
                ('host1',
                 salt.utils.odict.OrderedDict([
                     ('test_state0', {'result':  True}),
                     ('test_state', {'result': True}),
                 ])),
                ('host2',
                 salt.utils.odict.OrderedDict([
                     ('test_state0', {'result':  True}),
                     ('test_state', {'result': True}),
                 ])),
            ]),
            'test4': salt.utils.odict.OrderedDict([
                ('asup', salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('test_state0', {'result':  True}),
                         ('test_state', {'result': True}),
                     ])),
                    ('host2',
                     salt.utils.odict.OrderedDict([
                         ('test_state0', {'result':  True}),
                         ('test_state', {'result': True}),
                     ]))
                ]))
            ]),
            'test2': salt.utils.odict.OrderedDict([
                ('host1',
                 salt.utils.odict.OrderedDict([
                     ('test_state0', {'result':  None}),
                     ('test_state', {'result': True}),
                 ])),
                ('host2',
                 salt.utils.odict.OrderedDict([
                     ('test_state0', {'result':  True}),
                     ('test_state', {'result': 'abc'}),
                 ]))
            ])
        }
        for test, data in six.iteritems(test_valid_true_states):
            self.assertTrue(
                salt.utils.state.check_result(data),
                msg='{0} failed'.format(test))
        test_invalid_true_ht_states = {
            'test_onfail_simple2': (
                salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('test_vstate0', {'result':  False}),
                         ('test_vstate1', {'result': True}),
                     ])),
                ]),
                {
                    'test_vstate0': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                'run',
                                {'order': 10002}]},
                    'test_vstate1': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                salt.utils.odict.OrderedDict([
                                    ('onfail_stop', True),
                                    ('onfail',
                                     [salt.utils.odict.OrderedDict([('cmd', 'test_vstate0')])])
                                ]),
                                'run',
                                {'order': 10004}]},
                }
            ),
            'test_onfail_integ2': (
                salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('t_|-test_ivstate0_|-echo_|-run', {
                             'result':  False}),
                         ('cmd_|-test_ivstate0_|-echo_|-run', {
                             'result':  False}),
                         ('cmd_|-test_ivstate1_|-echo_|-run', {
                             'result': False}),
                     ])),
                ]),
                {
                    'test_ivstate0': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                'run',
                                {'order': 10002}],
                        't': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                              'run',
                              {'order': 10002}]},
                    'test_ivstate1': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                salt.utils.odict.OrderedDict([
                                    ('onfail_stop', False),
                                    ('onfail',
                                     [salt.utils.odict.OrderedDict([('cmd', 'test_ivstate0')])])
                                ]),
                                'run',
                                {'order': 10004}]},
                }
            ),
            'test_onfail_integ3': (
                salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('t_|-test_ivstate0_|-echo_|-run', {
                             'result':  True}),
                         ('cmd_|-test_ivstate0_|-echo_|-run', {
                             'result': False}),
                         ('cmd_|-test_ivstate1_|-echo_|-run', {
                             'result': False}),
                     ])),
                ]),
                {
                    'test_ivstate0': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                'run',
                                {'order': 10002}],
                        't': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                              'run',
                              {'order': 10002}]},
                    'test_ivstate1': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                salt.utils.odict.OrderedDict([
                                    ('onfail_stop', False),
                                    ('onfail',
                                     [salt.utils.odict.OrderedDict([('cmd', 'test_ivstate0')])])
                                ]),
                                'run',
                                {'order': 10004}]},
                }
            ),
            'test_onfail_integ4': (
                salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('t_|-test_ivstate0_|-echo_|-run', {
                             'result':  False}),
                         ('cmd_|-test_ivstate0_|-echo_|-run', {
                             'result': False}),
                         ('cmd_|-test_ivstate1_|-echo_|-run', {
                             'result': True}),
                     ])),
                ]),
                {
                    'test_ivstate0': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                'run',
                                {'order': 10002}],
                        't': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                              'run',
                              {'order': 10002}]},
                    'test_ivstate1': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                salt.utils.odict.OrderedDict([
                                    ('onfail_stop', False),
                                    ('onfail',
                                     [salt.utils.odict.OrderedDict([('cmd', 'test_ivstate0')])])
                                ]),
                                'run',
                                {'order': 10004}]},
                    'test_ivstate2': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                salt.utils.odict.OrderedDict([
                                    ('onfail_stop', True),
                                    ('onfail',
                                     [salt.utils.odict.OrderedDict([('cmd', 'test_ivstate0')])])
                                ]),
                                'run',
                                {'order': 10004}]},
                }
            ),
            'test_onfail': (
                salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('test_state0', {'result':  False}),
                         ('test_state', {'result': True}),
                     ])),
                ]),
                None
            ),
            'test_onfail_d': (
                salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('test_state0', {'result':  False}),
                         ('test_state', {'result': True}),
                     ])),
                ]),
                {}
            )
        }
        for test, testdata in six.iteritems(test_invalid_true_ht_states):
            data, ht = testdata
            for t_ in [a for a in data['host1']]:
                tdata = data['host1'][t_]
                if '_|-' in t_:
                    t_ = t_.split('_|-')[1]
                tdata['__id__'] = t_
            self.assertFalse(
                salt.utils.state.check_result(data, highstate=ht),
                msg='{0} failed'.format(test))

        test_valid_true_ht_states = {
            'test_onfail_integ': (
                salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('cmd_|-test_ivstate0_|-echo_|-run', {
                             'result':  False}),
                         ('cmd_|-test_ivstate1_|-echo_|-run', {
                             'result': True}),
                     ])),
                ]),
                {
                    'test_ivstate0': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                'run',
                                {'order': 10002}]},
                    'test_ivstate1': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                salt.utils.odict.OrderedDict([
                                    ('onfail_stop', False),
                                    ('onfail',
                                     [salt.utils.odict.OrderedDict([('cmd', 'test_ivstate0')])])
                                ]),
                                'run',
                                {'order': 10004}]},
                }
            ),
            'test_onfail_intega3': (
                salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('t_|-test_ivstate0_|-echo_|-run', {
                             'result':  True}),
                         ('cmd_|-test_ivstate0_|-echo_|-run', {
                             'result': False}),
                         ('cmd_|-test_ivstate1_|-echo_|-run', {
                             'result': True}),
                     ])),
                ]),
                {
                    'test_ivstate0': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                'run',
                                {'order': 10002}],
                        't': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                              'run',
                              {'order': 10002}]},
                    'test_ivstate1': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                salt.utils.odict.OrderedDict([
                                    ('onfail_stop', False),
                                    ('onfail',
                                     [salt.utils.odict.OrderedDict([('cmd', 'test_ivstate0')])])
                                ]),
                                'run',
                                {'order': 10004}]},
                }
            ),
            'test_onfail_simple': (
                salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('test_vstate0', {'result':  False}),
                         ('test_vstate1', {'result': True}),
                     ])),
                ]),
                {
                    'test_vstate0': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                'run',
                                {'order': 10002}]},
                    'test_vstate1': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                salt.utils.odict.OrderedDict([
                                    ('onfail_stop', False),
                                    ('onfail',
                                     [salt.utils.odict.OrderedDict([('cmd', 'test_vstate0')])])
                                ]),
                                'run',
                                {'order': 10004}]},
                }
            ),  # order is different
            'test_onfail_simple_rev': (
                salt.utils.odict.OrderedDict([
                    ('host1',
                     salt.utils.odict.OrderedDict([
                         ('test_vstate0', {'result':  False}),
                         ('test_vstate1', {'result': True}),
                     ])),
                ]),
                {
                    'test_vstate0': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                'run',
                                {'order': 10002}]},
                    'test_vstate1': {
                        '__env__': 'base',
                        '__sls__': 'a',
                        'cmd': [salt.utils.odict.OrderedDict([('name', '/bin/true')]),
                                salt.utils.odict.OrderedDict([
                                    ('onfail',
                                     [salt.utils.odict.OrderedDict([('cmd', 'test_vstate0')])])
                                ]),
                                salt.utils.odict.OrderedDict([('onfail_stop', False)]),
                                'run',
                                {'order': 10004}]},
                }
            )
        }
        for test, testdata in six.iteritems(test_valid_true_ht_states):
            data, ht = testdata
            for t_ in [a for a in data['host1']]:
                tdata = data['host1'][t_]
                if '_|-' in t_:
                    t_ = t_.split('_|-')[1]
                tdata['__id__'] = t_
            self.assertTrue(
                salt.utils.state.check_result(data, highstate=ht),
                msg='{0} failed'.format(test))
        test_valid_false_state = {'host1': {'test_state': {'result': False}}}
        self.assertFalse(salt.utils.state.check_result(test_valid_false_state))


class UtilStateMergeSubreturnTestcase(TestCase):
    '''
    Test cases for salt.utils.state.merge_subreturn function.
    '''
    main_ret = {
        'name': 'primary',
        # result may be missing, as primarysalt.utils.state is still in progress
        'comment': '',
        'changes': {},
    }
    sub_ret = {
        'name': 'secondary',
        'result': True,
        'comment': '',
        'changes': {},
    }

    def test_merge_result(self):
        # result not created if not needed
        for no_effect_result in [True, None]:
            m = copy.deepcopy(self.main_ret)
            s = copy.deepcopy(self.sub_ret)
            s['result'] = no_effect_result
            res = salt.utils.state.merge_subreturn(m, s)
            self.assertNotIn('result', res)

        # False subresult is propagated to existing result
        for original_result in [True, None, False]:
            m = copy.deepcopy(self.main_ret)
            m['result'] = original_result
            s = copy.deepcopy(self.sub_ret)
            s['result'] = False
            res = salt.utils.state.merge_subreturn(m, s)
            self.assertFalse(res['result'])

        # False result cannot be overridden
        for any_result in [True, None, False]:
            m = copy.deepcopy(self.main_ret)
            m['result'] = False
            s = copy.deepcopy(self.sub_ret)
            s['result'] = any_result
            res = salt.utils.state.merge_subreturn(m, s)
            self.assertFalse(res['result'])

    def test_merge_changes(self):
        # The main changes dict should always already exist,
        # and there should always be a changes dict in the secondary.
        primary_changes = {'old': None, 'new': 'my_resource'}
        secondary_changes = {'old': None, 'new': ['alarm-1', 'alarm-2']}

        # No changes case
        m = copy.deepcopy(self.main_ret)
        s = copy.deepcopy(self.sub_ret)
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertDictEqual(res['changes'], {})

        # New changes don't get rid of existing changes
        m = copy.deepcopy(self.main_ret)
        m['changes'] = copy.deepcopy(primary_changes)
        s = copy.deepcopy(self.sub_ret)
        s['changes'] = copy.deepcopy(secondary_changes)
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertDictEqual(res['changes'], {
            'old': None,
            'new': 'my_resource',
            'secondary': secondary_changes,
        })

        # The subkey parameter is respected
        m = copy.deepcopy(self.main_ret)
        m['changes'] = copy.deepcopy(primary_changes)
        s = copy.deepcopy(self.sub_ret)
        s['changes'] = copy.deepcopy(secondary_changes)
        res = salt.utils.state.merge_subreturn(m, s, subkey='alarms')
        self.assertDictEqual(res['changes'], {
            'old': None,
            'new': 'my_resource',
            'alarms': secondary_changes,
        })

    def test_merge_pchanges(self):
        primary_pchanges = {'old': None, 'new': 'my_resource'}
        secondary_pchanges = {'old': None, 'new': ['alarm-1', 'alarm-2']}

        # Neither main nor sub pchanges case
        m = copy.deepcopy(self.main_ret)
        s = copy.deepcopy(self.sub_ret)
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertNotIn('pchanges', res)

        # No main pchanges, sub pchanges
        m = copy.deepcopy(self.main_ret)
        s = copy.deepcopy(self.sub_ret)
        s['pchanges'] = copy.deepcopy(secondary_pchanges)
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertDictEqual(res['pchanges'], {
            'secondary': secondary_pchanges
        })

        # Main pchanges, no sub pchanges
        m = copy.deepcopy(self.main_ret)
        m['pchanges'] = copy.deepcopy(primary_pchanges)
        s = copy.deepcopy(self.sub_ret)
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertDictEqual(res['pchanges'], primary_pchanges)

        # Both main and sub pchanges, new pchanges don't affect existing ones
        m = copy.deepcopy(self.main_ret)
        m['pchanges'] = copy.deepcopy(primary_pchanges)
        s = copy.deepcopy(self.sub_ret)
        s['pchanges'] = copy.deepcopy(secondary_pchanges)
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertDictEqual(res['pchanges'], {
            'old': None,
            'new': 'my_resource',
            'secondary': secondary_pchanges,
        })

        # The subkey parameter is respected
        m = copy.deepcopy(self.main_ret)
        m['pchanges'] = copy.deepcopy(primary_pchanges)
        s = copy.deepcopy(self.sub_ret)
        s['pchanges'] = copy.deepcopy(secondary_pchanges)
        res = salt.utils.state.merge_subreturn(m, s, subkey='alarms')
        self.assertDictEqual(res['pchanges'], {
            'old': None,
            'new': 'my_resource',
            'alarms': secondary_pchanges,
        })

    def test_merge_comments(self):
        main_comment_1 = 'First primary comment.'
        main_comment_2 = 'Second primary comment.'
        sub_comment_1 = 'First secondary comment,\nwhich spans two lines.'
        sub_comment_2 = 'Second secondary comment: {0}'.format(
            'some error\n  And a traceback',
        )
        final_comment = textwrap.dedent('''\
            First primary comment.
            Second primary comment.
            First secondary comment,
            which spans two lines.
            Second secondary comment: some error
              And a traceback
        '''.rstrip())

        # Joining two strings
        m = copy.deepcopy(self.main_ret)
        m['comment'] = main_comment_1 + '\n' + main_comment_2
        s = copy.deepcopy(self.sub_ret)
        s['comment'] = sub_comment_1 + '\n' + sub_comment_2
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertMultiLineEqual(res['comment'], final_comment)

        # Joining string and a list
        m = copy.deepcopy(self.main_ret)
        m['comment'] = main_comment_1 + '\n' + main_comment_2
        s = copy.deepcopy(self.sub_ret)
        s['comment'] = [sub_comment_1, sub_comment_2]
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertMultiLineEqual(res['comment'], final_comment)

        # For tests where output is a list,
        # also test that final joined output will match
        # Joining list and a string
        m = copy.deepcopy(self.main_ret)
        m['comment'] = [main_comment_1, main_comment_2]
        s = copy.deepcopy(self.sub_ret)
        s['comment'] = sub_comment_1 + '\n' + sub_comment_2
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertEqual(res['comment'], [
            main_comment_1,
            main_comment_2,
            sub_comment_1 + '\n' + sub_comment_2,
        ])
        self.assertMultiLineEqual('\n'.join(res['comment']), final_comment)

        # Joining two lists
        m = copy.deepcopy(self.main_ret)
        m['comment'] = [main_comment_1, main_comment_2]
        s = copy.deepcopy(self.sub_ret)
        s['comment'] = [sub_comment_1, sub_comment_2]
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertEqual(res['comment'], [
            main_comment_1,
            main_comment_2,
            sub_comment_1,
            sub_comment_2,
        ])
        self.assertMultiLineEqual('\n'.join(res['comment']), final_comment)

    def test_merge_empty_comments(self):
        # Since the primarysalt.utils.state is in progress,
        # the main comment may be empty, either '' or [].
        # Note that [''] is a degenerate case and should never happen,
        # hence the behavior is left unspecified in that case.
        # The secondary comment should never be empty,
        # because thatsalt.utils.state has already returned,
        # so we leave the behavior unspecified in that case.
        sub_comment_1 = 'Secondary comment about changes:'
        sub_comment_2 = 'A diff that goes with the previous comment'
        # No contributions from primary
        final_comment = sub_comment_1 + '\n' + sub_comment_2

        # Joining empty string and a string
        m = copy.deepcopy(self.main_ret)
        m['comment'] = ''
        s = copy.deepcopy(self.sub_ret)
        s['comment'] = sub_comment_1 + '\n' + sub_comment_2
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertEqual(res['comment'], final_comment)

        # Joining empty string and a list
        m = copy.deepcopy(self.main_ret)
        m['comment'] = ''
        s = copy.deepcopy(self.sub_ret)
        s['comment'] = [sub_comment_1, sub_comment_2]
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertEqual(res['comment'], final_comment)

        # For tests where output is a list,
        # also test that final joined output will match
        # Joining empty list and a string
        m = copy.deepcopy(self.main_ret)
        m['comment'] = []
        s = copy.deepcopy(self.sub_ret)
        s['comment'] = sub_comment_1 + '\n' + sub_comment_2
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertEqual(res['comment'], [final_comment])
        self.assertEqual('\n'.join(res['comment']), final_comment)

        # Joining empty list and a list
        m = copy.deepcopy(self.main_ret)
        m['comment'] = []
        s = copy.deepcopy(self.sub_ret)
        s['comment'] = [sub_comment_1, sub_comment_2]
        res = salt.utils.state.merge_subreturn(m, s)
        self.assertEqual(res['comment'], [sub_comment_1, sub_comment_2])
        self.assertEqual('\n'.join(res['comment']), final_comment)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Anthony Shaw <anthonyshaw@apache.org>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)
import salt.modules.libcloud_dns as libcloud_dns


class MockDNSDriver(object):
    def __init__(self):
        pass


def get_mock_driver():
    return MockDNSDriver()


@skipIf(NO_MOCK, NO_MOCK_REASON)
class LibcloudDnsModuleTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        module_globals = {
            '_get_driver': MagicMock(return_value=MockDNSDriver()),
            '__salt__': {
                'config.option': MagicMock(return_value={
                    'test': {
                        'driver': 'test',
                        'key': '2orgk34kgk34g'
                    }
                })
            }
        }
        if libcloud_dns.HAS_LIBCLOUD is False:
            module_globals['sys.modules'] = {'libcloud': MagicMock()}

        return {libcloud_dns: module_globals}

    def test_module_creation(self):
        client = libcloud_dns._get_driver('test')
        self.assertFalse(client is None)

    def test_init(self):
        with patch('salt.utils.compat.pack_dunder', return_value=False) as dunder:
            libcloud_dns.__init__(None)
            dunder.assert_called_with('salt.modules.libcloud_dns')



# -*- coding: utf-8 -*-
'''
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import sys
import tempfile

# Import Salt Libs
import salt.utils.files
import salt.utils.platform
import salt.modules.cmdmod as cmdmod
from salt.exceptions import CommandExecutionError
from salt.log import LOG_LEVELS
from salt.ext.six.moves import builtins  # pylint: disable=import-error

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.paths import FILES
from tests.support.mock import (
    mock_open,
    Mock,
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON,
    patch
)

DEFAULT_SHELL = 'foo/bar'
MOCK_SHELL_FILE = '# List of acceptable shells\n' \
                  '\n'\
                  '/bin/bash\n'


class MockTimedProc(object):
    '''
    Class used as a stand-in for salt.utils.timed_subprocess.TimedProc
    '''
    class _Process(object):
        '''
        Used to provide a dummy "process" attribute
        '''
        def __init__(self, returncode=0, pid=12345):
            self.returncode = returncode
            self.pid = pid

    def __init__(self, stdout=None, stderr=None, returncode=0, pid=12345):
        if stdout is not None and not isinstance(stdout, bytes):
            raise TypeError('Must pass stdout to MockTimedProc as bytes')
        if stderr is not None and not isinstance(stderr, bytes):
            raise TypeError('Must pass stderr to MockTimedProc as bytes')
        self._stdout = stdout
        self._stderr = stderr
        self.process = self._Process(returncode=returncode, pid=pid)

    def run(self):
        pass

    @property
    def stdout(self):
        return self._stdout

    @property
    def stderr(self):
        return self._stderr


@skipIf(NO_MOCK, NO_MOCK_REASON)
class CMDMODTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Unit tests for the salt.modules.cmdmod module
    '''

    def setup_loader_modules(self):
        return {cmdmod: {}}

    @classmethod
    def setUpClass(cls):
        cls.mock_loglevels = {'info': 'foo', 'all': 'bar', 'critical': 'bar',
                              'trace': 'bar', 'garbage': 'bar', 'error': 'bar',
                              'debug': 'bar', 'warning': 'bar', 'quiet': 'bar'}

    @classmethod
    def tearDownClass(cls):
        del cls.mock_loglevels

    def test_render_cmd_no_template(self):
        '''
        Tests return when template=None
        '''
        self.assertEqual(cmdmod._render_cmd('foo', 'bar', None),
                         ('foo', 'bar'))

    def test_render_cmd_unavailable_engine(self):
        '''
        Tests CommandExecutionError raised when template isn't in the
        template registry
        '''
        self.assertRaises(CommandExecutionError,
                          cmdmod._render_cmd,
                          'boo', 'bar', 'baz')

    def test_check_loglevel_bad_level(self):
        '''
        Tests return of providing an invalid loglevel option
        '''
        with patch.dict(LOG_LEVELS, self.mock_loglevels):
            self.assertEqual(cmdmod._check_loglevel(level='bad_loglevel'), 'foo')

    def test_check_loglevel_bad_level_not_str(self):
        '''
        Tests the return of providing an invalid loglevel option that is not a string
        '''
        with patch.dict(LOG_LEVELS, self.mock_loglevels):
            self.assertEqual(cmdmod._check_loglevel(level=1000), 'foo')

    def test_check_loglevel_quiet(self):
        '''
        Tests the return of providing a loglevel of 'quiet'
        '''
        with patch.dict(LOG_LEVELS, self.mock_loglevels):
            self.assertEqual(cmdmod._check_loglevel(level='quiet'), None)

    def test_parse_env_not_env(self):
        '''
        Tests the return of an env that is not an env
        '''
        self.assertEqual(cmdmod._parse_env(None), {})

    def test_parse_env_list(self):
        '''
        Tests the return of an env that is a list
        '''
        ret = {'foo': None, 'bar': None}
        self.assertEqual(ret, cmdmod._parse_env(['foo', 'bar']))

    def test_parse_env_dict(self):
        '''
        Test the return of an env that is not a dict
        '''
        self.assertEqual(cmdmod._parse_env('test'), {})

    def test_run_shell_is_not_file(self):
        '''
        Tests error raised when shell is not available after _is_valid_shell error msg
        and os.path.isfile returns False
        '''
        with patch('salt.modules.cmdmod._is_valid_shell', MagicMock(return_value=True)):
            with patch('salt.utils.platform.is_windows', MagicMock(return_value=False)):
                with patch('os.path.isfile', MagicMock(return_value=False)):
                    self.assertRaises(CommandExecutionError, cmdmod._run, 'foo', 'bar')

    def test_run_shell_file_no_access(self):
        '''
        Tests error raised when shell is not available after _is_valid_shell error msg,
        os.path.isfile returns True, but os.access returns False
        '''
        with patch('salt.modules.cmdmod._is_valid_shell', MagicMock(return_value=True)):
            with patch('salt.utils.platform.is_windows', MagicMock(return_value=False)):
                with patch('os.path.isfile', MagicMock(return_value=True)):
                    with patch('os.access', MagicMock(return_value=False)):
                        self.assertRaises(CommandExecutionError, cmdmod._run, 'foo', 'bar')

    def test_run_runas_with_windows(self):
        '''
        Tests error raised when runas is passed on windows
        '''
        with patch('salt.modules.cmdmod._is_valid_shell', MagicMock(return_value=True)):
            with patch('salt.utils.platform.is_windows', MagicMock(return_value=True)):
                with patch.dict(cmdmod.__grains__, {'os': 'fake_os'}):
                    self.assertRaises(CommandExecutionError,
                                      cmdmod._run,
                                      'foo', 'bar', runas='baz')

    def test_run_user_not_available(self):
        '''
        Tests return when runas user is not available
        '''
        with patch('salt.modules.cmdmod._is_valid_shell', MagicMock(return_value=True)):
            with patch('os.path.isfile', MagicMock(return_value=True)):
                with patch('os.access', MagicMock(return_value=True)):
                    self.assertRaises(CommandExecutionError, cmdmod._run, 'foo', 'bar', runas='baz')

    def test_run_zero_umask(self):
        '''
        Tests error raised when umask is set to zero
        '''
        with patch('salt.modules.cmdmod._is_valid_shell', MagicMock(return_value=True)):
            with patch('salt.utils.platform.is_windows', MagicMock(return_value=False)):
                with patch('os.path.isfile', MagicMock(return_value=True)):
                    with patch('os.access', MagicMock(return_value=True)):
                        self.assertRaises(CommandExecutionError, cmdmod._run, 'foo', 'bar', umask=0)

    def test_run_invalid_umask(self):
        '''
        Tests error raised when an invalid umask is given
        '''
        with patch('salt.modules.cmdmod._is_valid_shell', MagicMock(return_value=True)):
            with patch('salt.utils.platform.is_windows', MagicMock(return_value=False)):
                with patch('os.path.isfile', MagicMock(return_value=True)):
                    with patch('os.access', MagicMock(return_value=True)):
                        self.assertRaises(CommandExecutionError, cmdmod._run, 'foo', 'bar', umask='baz')

    def test_run_invalid_cwd_not_abs_path(self):
        '''
        Tests error raised when cwd is not an absolute path
        '''
        with patch('salt.modules.cmdmod._is_valid_shell', MagicMock(return_value=True)):
            with patch('salt.utils.platform.is_windows', MagicMock(return_value=False)):
                with patch('os.path.isfile', MagicMock(return_value=True)):
                    with patch('os.access', MagicMock(return_value=True)):
                        self.assertRaises(CommandExecutionError, cmdmod._run, 'foo', 'bar')

    def test_run_invalid_cwd_not_dir(self):
        '''
        Tests error raised when cwd is not a dir
        '''
        with patch('salt.modules.cmdmod._is_valid_shell', MagicMock(return_value=True)):
            with patch('salt.utils.platform.is_windows', MagicMock(return_value=False)):
                with patch('os.path.isfile', MagicMock(return_value=True)):
                    with patch('os.access', MagicMock(return_value=True)):
                        with patch('os.path.isabs', MagicMock(return_value=True)):
                            self.assertRaises(CommandExecutionError, cmdmod._run, 'foo', 'bar')

    def test_run_no_vt_os_error(self):
        '''
        Tests error raised when not useing vt and OSError is provided
        '''
        with patch('salt.modules.cmdmod._is_valid_shell', MagicMock(return_value=True)):
            with patch('salt.utils.platform.is_windows', MagicMock(return_value=False)):
                with patch('os.path.isfile', MagicMock(return_value=True)):
                    with patch('os.access', MagicMock(return_value=True)):
                        with patch('salt.utils.timed_subprocess.TimedProc', MagicMock(side_effect=OSError)):
                            self.assertRaises(CommandExecutionError, cmdmod._run, 'foo')

    def test_run_no_vt_io_error(self):
        '''
        Tests error raised when not useing vt and IOError is provided
        '''
        with patch('salt.modules.cmdmod._is_valid_shell', MagicMock(return_value=True)):
            with patch('salt.utils.platform.is_windows', MagicMock(return_value=False)):
                with patch('os.path.isfile', MagicMock(return_value=True)):
                    with patch('os.access', MagicMock(return_value=True)):
                        with patch('salt.utils.timed_subprocess.TimedProc', MagicMock(side_effect=IOError)):
                            self.assertRaises(CommandExecutionError, cmdmod._run, 'foo')

    @skipIf(salt.utils.platform.is_windows(), 'Do not run on Windows')
    @skipIf(True, 'Test breaks unittests runs')
    def test_run(self):
        '''
        Tests end result when a command is not found
        '''
        with patch('salt.modules.cmdmod._is_valid_shell', MagicMock(return_value=True)):
            with patch('salt.utils.platform.is_windows', MagicMock(return_value=False)):
                with patch('os.path.isfile', MagicMock(return_value=True)):
                    with patch('os.access', MagicMock(return_value=True)):
                        ret = cmdmod._run('foo', cwd=os.getcwd(), use_vt=True).get('stderr')
                        self.assertIn('foo', ret)

    def test_is_valid_shell_windows(self):
        '''
        Tests return if running on windows
        '''
        with patch('salt.utils.platform.is_windows', MagicMock(return_value=True)):
            self.assertTrue(cmdmod._is_valid_shell('foo'))

    @skipIf(salt.utils.platform.is_windows(), 'Do not run on Windows')
    def test_is_valid_shell_none(self):
        '''
        Tests return of when os.path.exists(/etc/shells) isn't available
        '''
        with patch('os.path.exists', MagicMock(return_value=False)):
            self.assertIsNone(cmdmod._is_valid_shell('foo'))

    def test_is_valid_shell_available(self):
        '''
        Tests return when provided shell is available
        '''
        with patch('os.path.exists', MagicMock(return_value=True)):
            with patch('salt.utils.files.fopen', mock_open(read_data=MOCK_SHELL_FILE)):
                self.assertTrue(cmdmod._is_valid_shell('/bin/bash'))

    @skipIf(salt.utils.platform.is_windows(), 'Do not run on Windows')
    def test_is_valid_shell_unavailable(self):
        '''
        Tests return when provided shell is not available
        '''
        with patch('os.path.exists', MagicMock(return_value=True)):
            with patch('salt.utils.files.fopen', mock_open(read_data=MOCK_SHELL_FILE)):
                self.assertFalse(cmdmod._is_valid_shell('foo'))

    @skipIf(salt.utils.platform.is_windows(), 'Do not run on Windows')
    def test_os_environment_remains_intact(self):
        '''
        Make sure the OS environment is not tainted after running a command
        that specifies runas.
        '''
        with patch('pwd.getpwnam') as getpwnam_mock:
            with patch('subprocess.Popen') as popen_mock:
                environment = os.environ.copy()

                popen_mock.return_value = Mock(
                    communicate=lambda *args, **kwags: [b'', None],
                    pid=lambda: 1,
                    retcode=0
                )

                with patch.dict(cmdmod.__grains__, {'os': 'Darwin', 'os_family': 'Solaris'}):
                    if sys.platform.startswith(('freebsd', 'openbsd')):
                        shell = '/bin/sh'
                    else:
                        shell = '/bin/bash'

                    cmdmod._run('ls',
                                cwd=tempfile.gettempdir(),
                                runas='foobar',
                                shell=shell)

                    environment2 = os.environ.copy()

                    self.assertEqual(environment, environment2)

                    if not salt.utils.platform.is_darwin():
                        getpwnam_mock.assert_called_with('foobar')

    def test_run_cwd_doesnt_exist_issue_7154(self):
        '''
        cmd.run should fail and raise
        salt.exceptions.CommandExecutionError if the cwd dir does not
        exist
        '''
        cmd = 'echo OHAI'
        cwd = '/path/to/nowhere'
        try:
            cmdmod.run_all(cmd, cwd=cwd)
        except CommandExecutionError:
            pass
        else:
            raise RuntimeError

    def test_run_all_binary_replace(self):
        '''
        Test for failed decoding of binary data, for instance when doing
        something silly like using dd to read from /dev/urandom and write to
        /dev/stdout.
        '''
        # Since we're using unicode_literals, read the random bytes from a file
        rand_bytes_file = os.path.join(FILES, 'file', 'base', 'random_bytes')
        with salt.utils.files.fopen(rand_bytes_file, 'rb') as fp_:
            stdout_bytes = fp_.read()

        # stdout with the non-decodable bits replaced with the unicode
        # replacement character U+FFFD.
        stdout_unicode = '\ufffd\x1b\ufffd\ufffd\n'
        stderr_bytes = b'1+0 records in\n1+0 records out\n' \
                       b'4 bytes copied, 9.1522e-05 s, 43.7 kB/s\n'
        stderr_unicode = stderr_bytes.decode()

        proc = MagicMock(
            return_value=MockTimedProc(
                stdout=stdout_bytes,
                stderr=stderr_bytes
            )
        )
        with patch('salt.utils.timed_subprocess.TimedProc', proc):
            ret = cmdmod.run_all(
                'dd if=/dev/urandom of=/dev/stdout bs=4 count=1',
                rstrip=False)

        self.assertEqual(ret['stdout'], stdout_unicode)
        self.assertEqual(ret['stderr'], stderr_unicode)

    def test_run_all_none(self):
        '''
        Tests cases when proc.stdout or proc.stderr are None. These should be
        caught and replaced with empty strings.
        '''
        proc = MagicMock(return_value=MockTimedProc(stdout=None, stderr=None))
        with patch('salt.utils.timed_subprocess.TimedProc', proc):
            ret = cmdmod.run_all('some command', rstrip=False)

        self.assertEqual(ret['stdout'], '')
        self.assertEqual(ret['stderr'], '')

    def test_run_all_unicode(self):
        '''
        Ensure that unicode stdout and stderr are decoded properly
        '''
        stdout_unicode = 'Here is some unicode: спам'
        stderr_unicode = 'Here is some unicode: яйца'
        stdout_bytes = stdout_unicode.encode('utf-8')
        stderr_bytes = stderr_unicode.encode('utf-8')

        proc = MagicMock(
            return_value=MockTimedProc(
                stdout=stdout_bytes,
                stderr=stderr_bytes
            )
        )

        with patch('salt.utils.timed_subprocess.TimedProc', proc), \
                patch.object(builtins, '__salt_system_encoding__', 'utf-8'):
            ret = cmdmod.run_all('some command', rstrip=False)

        self.assertEqual(ret['stdout'], stdout_unicode)
        self.assertEqual(ret['stderr'], stderr_unicode)

    def test_run_all_output_encoding(self):
        '''
        Test that specifying the output encoding works as expected
        '''
        stdout = 'Æ'
        stdout_latin1_enc = stdout.encode('latin1')

        proc = MagicMock(return_value=MockTimedProc(stdout=stdout_latin1_enc))

        with patch('salt.utils.timed_subprocess.TimedProc', proc), \
                patch.object(builtins, '__salt_system_encoding__', 'utf-8'):
            ret = cmdmod.run_all('some command', output_encoding='latin1')

        self.assertEqual(ret['stdout'], stdout)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.unit.modules.archive_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os.path

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import salt libs
import salt.modules.archive as archive
import salt.utils.path
from salt.exceptions import CommandNotFoundError

# Import 3rd-party libs
from salt.ext.six.moves import zip


class ZipFileMock(MagicMock):
    def __init__(self, files=None, **kwargs):  # pylint: disable=W0231
        if files is None:
            files = ['salt']
        MagicMock.__init__(self, **kwargs)
        self._files = files
        self.external_attr = 0o0777 << 16

    def namelist(self):
        return self._files


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ArchiveTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {archive: {'__grains__': {'id': 0}}}

    def test_tar(self):
        with patch('glob.glob', lambda pathname: [pathname]):
            with patch('salt.utils.path.which', lambda exe: exe):
                mock = MagicMock(return_value='salt')
                with patch.dict(archive.__salt__, {'cmd.run': mock}):
                    ret = archive.tar(
                        '-zcvf', 'foo.tar',
                        ['/tmp/something-to-compress-1',
                         '/tmp/something-to-compress-2'],
                        template=None
                    )
                    self.assertEqual(['salt'], ret)
                    mock.assert_called_once_with(
                        ['tar', '-zcvf', 'foo.tar', '/tmp/something-to-compress-1',
                         '/tmp/something-to-compress-2'],
                        runas=None, python_shell=False, template=None, cwd=None
                    )

                mock = MagicMock(return_value='salt')
                with patch.dict(archive.__salt__, {'cmd.run': mock}):
                    ret = archive.tar(
                        '-zcvf', 'foo.tar',
                        '/tmp/something-to-compress-1,/tmp/something-to-compress-2',
                        template=None
                    )
                    self.assertEqual(['salt'], ret)
                    mock.assert_called_once_with(
                        ['tar', '-zcvf', 'foo.tar', '/tmp/something-to-compress-1',
                         '/tmp/something-to-compress-2'],
                        runas=None, python_shell=False, template=None, cwd=None
                    )

    def test_tar_raises_exception_if_not_found(self):
        with patch('salt.utils.path.which', lambda exe: None):
            mock = MagicMock(return_value='salt')
            with patch.dict(archive.__salt__, {'cmd.run': mock}):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.tar,
                    'zxvf',
                    'foo.tar',
                    '/tmp/something-to-compress'
                )
                self.assertFalse(mock.called)

    def test_gzip(self):
        mock = MagicMock(return_value='salt')
        with patch.dict(archive.__salt__, {'cmd.run': mock}):
            with patch('salt.utils.path.which', lambda exe: exe):
                ret = archive.gzip('/tmp/something-to-compress')
                self.assertEqual(['salt'], ret)
                mock.assert_called_once_with(
                    ['gzip', '/tmp/something-to-compress'],
                    runas=None, python_shell=False, template=None
                )

    def test_gzip_raises_exception_if_not_found(self):
        mock = MagicMock(return_value='salt')
        with patch.dict(archive.__salt__, {'cmd.run': mock}):
            with patch('salt.utils.path.which', lambda exe: None):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.gzip, '/tmp/something-to-compress'
                )
                self.assertFalse(mock.called)

    def test_gunzip(self):
        mock = MagicMock(return_value='salt')
        with patch.dict(archive.__salt__, {'cmd.run': mock}):
            with patch('salt.utils.path.which', lambda exe: exe):
                ret = archive.gunzip('/tmp/something-to-decompress.tar.gz')
                self.assertEqual(['salt'], ret)
                mock.assert_called_once_with(
                    ['gunzip', '/tmp/something-to-decompress.tar.gz'],
                    runas=None, python_shell=False, template=None
                )

    def test_gunzip_raises_exception_if_not_found(self):
        mock = MagicMock(return_value='salt')
        with patch.dict(archive.__salt__, {'cmd.run': mock}):
            with patch('salt.utils.path.which', lambda exe: None):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.gunzip,
                    '/tmp/something-to-decompress.tar.gz'
                )
                self.assertFalse(mock.called)

    def test_cmd_zip(self):
        with patch('glob.glob', lambda pathname: [pathname]):
            with patch('salt.utils.path.which', lambda exe: exe):
                mock = MagicMock(return_value='salt')
                with patch.dict(archive.__salt__, {'cmd.run': mock}):
                    ret = archive.cmd_zip(
                        '/tmp/salt.{{grains.id}}.zip',
                        '/tmp/tmpePe8yO,/tmp/tmpLeSw1A',
                        template='jinja'
                    )
                    self.assertEqual(['salt'], ret)
                    mock.assert_called_once_with(
                        ['zip', '-r', '/tmp/salt.{{grains.id}}.zip',
                         '/tmp/tmpePe8yO', '/tmp/tmpLeSw1A'],
                         runas=None, python_shell=False, template='jinja', cwd=None
                    )

                mock = MagicMock(return_value='salt')
                with patch.dict(archive.__salt__, {'cmd.run': mock}):
                    ret = archive.cmd_zip(
                        '/tmp/salt.{{grains.id}}.zip',
                        ['/tmp/tmpePe8yO', '/tmp/tmpLeSw1A'],
                        template='jinja'
                    )
                    self.assertEqual(['salt'], ret)
                    mock.assert_called_once_with(
                        ['zip', '-r', '/tmp/salt.{{grains.id}}.zip',
                         '/tmp/tmpePe8yO', '/tmp/tmpLeSw1A'],
                        runas=None, python_shell=False, template='jinja', cwd=None
                    )

    def test_zip(self):
        with patch('glob.glob', lambda pathname: [pathname]):
            with patch.multiple(os.path, **{'isdir': MagicMock(return_value=False),
                                            'exists': MagicMock(return_value=True)}):
                with patch('zipfile.ZipFile', MagicMock()):
                    ret = archive.zip_(
                        '/tmp/salt.{{grains.id}}.zip',
                        '/tmp/tmpePe8yO,/tmp/tmpLeSw1A',
                        template='jinja'
                    )
                    expected = [os.path.join('tmp', 'tmpePe8yO'),
                                os.path.join('tmp', 'tmpLeSw1A')]
                    self.assertEqual(expected, ret)

    def test_zip_raises_exception_if_not_found(self):
        mock = MagicMock(return_value='salt')
        with patch.dict(archive.__salt__, {'cmd.run': mock}):
            with patch('salt.utils.path.which', lambda exe: None):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.cmd_zip,
                    '/tmp/salt.{{grains.id}}.zip',
                    '/tmp/tmpePe8yO,/tmp/tmpLeSw1A',
                    template='jinja',
                )
                self.assertFalse(mock.called)

    def test_cmd_unzip(self):
        def _get_mock():
            '''
            Create a new MagicMock for each scenario in this test, so that
            assert_called_once_with doesn't complain that the same mock object
            is called more than once.
            '''
            return MagicMock(return_value={'stdout': 'salt',
                                           'stderr': '',
                                           'pid': 12345,
                                           'retcode': 0})

        with patch('salt.utils.path.which', lambda exe: exe):

            mock = _get_mock()
            with patch.dict(archive.__salt__, {'cmd.run_all': mock}):
                ret = archive.cmd_unzip(
                    '/tmp/salt.{{grains.id}}.zip',
                    '/tmp/dest',
                    excludes='/tmp/tmpePe8yO,/tmp/tmpLeSw1A',
                    template='jinja'
                )
                self.assertEqual(['salt'], ret)
                mock.assert_called_once_with(
                    ['unzip', '/tmp/salt.{{grains.id}}.zip', '-d', '/tmp/dest',
                     '-x', '/tmp/tmpePe8yO', '/tmp/tmpLeSw1A'],
                    output_loglevel='debug',
                    python_shell=False,
                    redirect_stderr=True,
                    runas=None,
                    template='jinja'
                )

            mock = _get_mock()
            with patch.dict(archive.__salt__, {'cmd.run_all': mock}):
                ret = archive.cmd_unzip(
                    '/tmp/salt.{{grains.id}}.zip',
                    '/tmp/dest',
                    excludes=['/tmp/tmpePe8yO', '/tmp/tmpLeSw1A'],
                    template='jinja'
                )
                self.assertEqual(['salt'], ret)
                mock.assert_called_once_with(
                    ['unzip', '/tmp/salt.{{grains.id}}.zip', '-d', '/tmp/dest',
                     '-x', '/tmp/tmpePe8yO', '/tmp/tmpLeSw1A'],
                    output_loglevel='debug',
                    python_shell=False,
                    redirect_stderr=True,
                    runas=None,
                    template='jinja'
                )

            mock = _get_mock()
            with patch.dict(archive.__salt__, {'cmd.run_all': mock}):
                ret = archive.cmd_unzip(
                    '/tmp/salt.{{grains.id}}.zip',
                    '/tmp/dest',
                    excludes='/tmp/tmpePe8yO,/tmp/tmpLeSw1A',
                    template='jinja',
                    options='-fo'
                )
                self.assertEqual(['salt'], ret)
                mock.assert_called_once_with(
                    ['unzip', '-fo', '/tmp/salt.{{grains.id}}.zip', '-d',
                     '/tmp/dest', '-x', '/tmp/tmpePe8yO', '/tmp/tmpLeSw1A'],
                    output_loglevel='debug',
                    python_shell=False,
                    redirect_stderr=True,
                    runas=None,
                    template='jinja'
                )

            mock = _get_mock()
            with patch.dict(archive.__salt__, {'cmd.run_all': mock}):
                ret = archive.cmd_unzip(
                    '/tmp/salt.{{grains.id}}.zip',
                    '/tmp/dest',
                    excludes=['/tmp/tmpePe8yO', '/tmp/tmpLeSw1A'],
                    template='jinja',
                    options='-fo'
                )
                self.assertEqual(['salt'], ret)
                mock.assert_called_once_with(
                    ['unzip', '-fo', '/tmp/salt.{{grains.id}}.zip', '-d',
                     '/tmp/dest', '-x', '/tmp/tmpePe8yO', '/tmp/tmpLeSw1A'],
                    output_loglevel='debug',
                    python_shell=False,
                    redirect_stderr=True,
                    runas=None,
                    template='jinja'
                )

            mock = _get_mock()
            with patch.dict(archive.__salt__, {'cmd.run_all': mock}):
                ret = archive.cmd_unzip(
                    '/tmp/salt.{{grains.id}}.zip',
                    '/tmp/dest',
                    excludes=['/tmp/tmpePe8yO', '/tmp/tmpLeSw1A'],
                    template='jinja',
                    options='-fo',
                    password='asdf',
                )
                self.assertEqual(['salt'], ret)
                mock.assert_called_once_with(
                    ['unzip', '-P', 'asdf', '-fo', '/tmp/salt.{{grains.id}}.zip',
                     '-d', '/tmp/dest', '-x', '/tmp/tmpePe8yO', '/tmp/tmpLeSw1A'],
                    output_loglevel='quiet',
                    python_shell=False,
                    redirect_stderr=True,
                    runas=None,
                    template='jinja'
                )

    def test_unzip(self):
        mock = ZipFileMock()
        with patch('zipfile.ZipFile', mock):
            ret = archive.unzip(
                '/tmp/salt.{{grains.id}}.zip',
                '/tmp/dest',
                excludes='/tmp/tmpePe8yO,/tmp/tmpLeSw1A',
                template='jinja',
                extract_perms=False
            )
            self.assertEqual(['salt'], ret)

    def test_unzip_raises_exception_if_not_found(self):
        mock = MagicMock(return_value='salt')
        with patch.dict(archive.__salt__, {'cmd.run': mock}):
            with patch('salt.utils.path.which', lambda exe: None):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.cmd_unzip,
                    '/tmp/salt.{{grains.id}}.zip',
                    '/tmp/dest',
                    excludes='/tmp/tmpePe8yO,/tmp/tmpLeSw1A',
                    template='jinja',
                )
                self.assertFalse(mock.called)

    def test_rar(self):
        with patch('glob.glob', lambda pathname: [pathname]):
            with patch('salt.utils.path.which', lambda exe: exe):
                mock = MagicMock(return_value='salt')
                with patch.dict(archive.__salt__, {'cmd.run': mock}):
                    ret = archive.rar(
                        '/tmp/rarfile.rar',
                        '/tmp/sourcefile1,/tmp/sourcefile2'
                    )
                    self.assertEqual(['salt'], ret)
                    mock.assert_called_once_with(
                        ['rar', 'a', '-idp', '/tmp/rarfile.rar',
                         '/tmp/sourcefile1', '/tmp/sourcefile2'],
                        runas=None, python_shell=False, template=None, cwd=None
                    )

                mock = MagicMock(return_value='salt')
                with patch.dict(archive.__salt__, {'cmd.run': mock}):
                    ret = archive.rar(
                        '/tmp/rarfile.rar',
                        ['/tmp/sourcefile1', '/tmp/sourcefile2']
                    )
                    self.assertEqual(['salt'], ret)
                    mock.assert_called_once_with(
                        ['rar', 'a', '-idp', '/tmp/rarfile.rar',
                         '/tmp/sourcefile1', '/tmp/sourcefile2'],
                        runas=None, python_shell=False, template=None, cwd=None
                    )

    def test_rar_raises_exception_if_not_found(self):
        mock = MagicMock(return_value='salt')
        with patch.dict(archive.__salt__, {'cmd.run': mock}):
            with patch('salt.utils.path.which', lambda exe: None):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.rar,
                    '/tmp/rarfile.rar',
                    '/tmp/sourcefile1,/tmp/sourcefile2'
                )
                self.assertFalse(mock.called)

    @skipIf(salt.utils.path.which_bin(('unrar', 'rar')) is None, 'unrar not installed')
    def test_unrar(self):
        with patch('salt.utils.path.which_bin', lambda exe: exe[0] if isinstance(exe, (list, tuple)) else exe):
            with patch('salt.utils.path.which', lambda exe: exe[0] if isinstance(exe, (list, tuple)) else exe):
                mock = MagicMock(return_value='salt')
                with patch.dict(archive.__salt__, {'cmd.run': mock}):
                    ret = archive.unrar(
                        '/tmp/rarfile.rar',
                        '/home/strongbad/',
                        excludes='file_1,file_2'
                    )
                    self.assertEqual(['salt'], ret)
                    mock.assert_called_once_with(
                        ['unrar', 'x', '-idp', '/tmp/rarfile.rar',
                         '-x', 'file_1', '-x', 'file_2', '/home/strongbad/'],
                        runas=None, python_shell=False, template=None
                    )

                mock = MagicMock(return_value='salt')
                with patch.dict(archive.__salt__, {'cmd.run': mock}):
                    ret = archive.unrar(
                        '/tmp/rarfile.rar',
                        '/home/strongbad/',
                        excludes=['file_1', 'file_2']
                    )
                    self.assertEqual(['salt'], ret)
                    mock.assert_called_once_with(
                        ['unrar', 'x', '-idp', '/tmp/rarfile.rar',
                         '-x', 'file_1', '-x', 'file_2', '/home/strongbad/'],
                        runas=None, python_shell=False, template=None
                    )

    def test_unrar_raises_exception_if_not_found(self):
        with patch('salt.utils.path.which_bin', lambda exe: None):
            mock = MagicMock(return_value='salt')
            with patch.dict(archive.__salt__, {'cmd.run': mock}):
                self.assertRaises(
                    CommandNotFoundError,
                    archive.unrar,
                    '/tmp/rarfile.rar',
                    '/home/strongbad/',
                )
                self.assertFalse(mock.called)

    def test_trim_files(self):
        with patch('salt.utils.path.which_bin', lambda exe: exe):
            source = 'file.tar.gz'
            tmp_dir = 'temp'
            files = [
                '\n'.join(['file1'] * 200),
                '\n'.join(['file2'] * 200),
                'this\nthat\nother',
                'this\nthat\nother',
                'this\nthat\nother'
            ]
            trim_opt = [
                True,
                False,
                3,
                1,
                0
            ]
            trim_100 = (['file1'] * 100)
            trim_100.append("List trimmed after 100 files.")
            expected = [
                trim_100,
                ['file2'] * 200,
                ['this', 'that', 'other'],
                ['this', 'List trimmed after 1 files.'],
                ['List trimmed after 0 files.'],
            ]

            for test_files, test_trim_opts, test_expected in zip(files, trim_opt, expected):
                with patch.dict(archive.__salt__, {'cmd.run': MagicMock(return_value=test_files)}):
                    ret = archive.unrar(source, tmp_dir, trim_output=test_trim_opts)
                    self.assertEqual(ret, test_expected)



# coding: utf-8

# Python libs
from __future__ import absolute_import

# Salt libs
from salt.beacons import twilio_txt_msg

# Salt testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import 3rd Party libs
try:
    import twilio
    # Grab version, ensure elements are ints
    twilio_version = tuple([int(x) for x in twilio.__version_info__])
    if twilio_version > (5, ):
        TWILIO_5 = False
    else:
        TWILIO_5 = True
    HAS_TWILIO = True
except ImportError:
    HAS_TWILIO = False

import logging
log = logging.getLogger(__name__)


class MockTwilioRestException(Exception):
    '''
    Mock TwilioRestException class
    '''
    def __init__(self):
        self.code = 'error code'
        self.msg = 'Exception error'
        self.status = 'Not send'
        super(MockTwilioRestException, self).__init__(self.msg)


class MockMessages(object):
    '''
    Mock SMS class
    '''
    flag = None

    def __init__(self):
        self.sid = '011'
        self.price = '200'
        self.price_unit = '1'
        self.status = 'Sent'
        self.num_segments = '2'
        self.num_media = '0'
        self.body = None
        self.date_sent = '01-01-2015'
        self.date_created = '01-01-2015'
        self.to = None
        self.from_ = None

    def create(self, body, to, from_):
        '''
        Mock create method
        '''
        msg = MockMessages()
        if self.flag == 1:
            raise MockTwilioRestException()
        msg.body = body
        msg.to = to
        msg.from_ = from_
        return msg

    def list(self, to):
        '''
        Mock list method
        '''
        msg = MockMessages()
        return [msg]

    def delete(self):
        '''
        Mock delete method
        '''
        return None


class MockSMS(object):
    '''
    Mock SMS class
    '''
    def __init__(self):
        self.messages = MockMessages()


class MockTwilioRestClient(object):
    '''
    Mock TwilioRestClient class
    '''
    def __init__(self):
        if TWILIO_5:
            self.sms = MockSMS()
        else:
            self.messages = MockMessages()


@skipIf(not HAS_TWILIO, 'twilio.rest is not available')
@skipIf(NO_MOCK, NO_MOCK_REASON)
class TwilioMsgTxtBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.twilio_txt_msg
    '''
    def setup_loader_modules(self):
        return {twilio_txt_msg: {}}

    def test_validate_dictionary_config(self):
        '''
        Test empty configuration
        '''
        config = {}
        ret = twilio_txt_msg.validate(config)
        self.assertEqual(ret, (False, ('Configuration for twilio_txt_msg '
                                       'beacon must be a list.')))

    def test_validate_empty_config(self):
        '''
        Test empty configuration
        '''
        config = [{}]
        ret = twilio_txt_msg.validate(config)
        self.assertEqual(ret, (False, ('Configuration for twilio_txt_msg '
                                       'beacon must contain account_sid, '
                                       'auth_token and twilio_number items.')))

    def test_validate_missing_config_item(self):
        '''
        Test empty configuration
        '''
        config = [{'account_sid': 'ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX',
                   'twilio_number': '+15555555555'}]

        ret = twilio_txt_msg.validate(config)
        self.assertEqual(ret, (False, ('Configuration for twilio_txt_msg '
                                       'beacon must contain account_sid, '
                                       'auth_token and twilio_number items.')))

    def test_receive_message(self):
        '''
        Test receive a message
        '''
        config = [{'account_sid': 'ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX',
                   'auth_token': 'my_token',
                   'twilio_number': '+15555555555'}]

        ret = twilio_txt_msg.validate(config)
        self.assertEqual(ret, (True, 'Valid beacon configuration'))

        _expected_return = [{'texts': [{'body': 'None',
                                        'images': [],
                                        'from': 'None',
                                        'id': '011',
                                        'sent': '01-01-2015'}]}]
        mock = MagicMock(return_value=MockTwilioRestClient())
        with patch.object(twilio_txt_msg, 'TwilioRestClient', mock):
            ret = twilio_txt_msg.beacon(config)
        self.assertEqual(ret, _expected_return)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
'''

# Import Salt Libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import traceback

# Import Salt Testing Libs
from tests.support.case import ShellCase
from tests.support.mixins import RUNTIME_VARS

# Import Salt libs
import salt.config
import salt.utils.yaml
from salt.output import display_output
from salt.ext import six


class OutputReturnTest(ShellCase):
    '''
    Integration tests to ensure outputters return their expected format.
    Tests against situations where the loader might not be returning the
    right outputter even though it was explicitly requested.
    '''

    def test_output_json(self):
        '''
        Tests the return of json-formatted data
        '''
        ret = self.run_call('test.ping --out=json')
        self.assertIn('{', ret)
        self.assertIn('"local": true', ''.join(ret))
        self.assertIn('}', ''.join(ret))

    def test_output_nested(self):
        '''
        Tests the return of nested-formatted data
        '''
        expected = ['local:', '    True']
        ret = self.run_call('test.ping --out=nested')
        self.assertEqual(ret, expected)

    def test_output_quiet(self):
        '''
        Tests the return of an out=quiet query
        '''
        expected = []
        ret = self.run_call('test.ping --out=quiet')
        self.assertEqual(ret, expected)

    def test_output_pprint(self):
        '''
        Tests the return of pprint-formatted data
        '''
        expected = ["{u'local': True}"] if six.PY2 else ["{'local': True}"]
        ret = self.run_call('test.ping --out=pprint')
        self.assertEqual(ret, expected)

    def test_output_raw(self):
        '''
        Tests the return of raw-formatted data
        '''
        expected = ["{u'local': True}"] if six.PY2 else ["{'local': True}"]
        ret = self.run_call('test.ping --out=raw')
        self.assertEqual(ret, expected)

    def test_output_txt(self):
        '''
        Tests the return of txt-formatted data
        '''
        expected = ['local: True']
        ret = self.run_call('test.ping --out=txt')
        self.assertEqual(ret, expected)

    def test_output_yaml(self):
        '''
        Tests the return of yaml-formatted data
        '''
        expected = ['local: true']
        ret = self.run_call('test.ping --out=yaml')
        self.assertEqual(ret, expected)

    def test_output_yaml_namespaced_dict_wrapper(self):
        '''
        Tests the ability to dump a NamespacedDictWrapper instance, as used in
        magic dunders like __grains__ and __pillar__

        See https://github.com/saltstack/salt/issues/49269
        '''
        dumped_yaml = '\n'.join(self.run_call('grains.items --out=yaml'))
        loaded_yaml = salt.utils.yaml.safe_load(dumped_yaml)
        # We just want to check that the dumped YAML loades as a dict with a
        # single top-level key, we don't care about the real contents.
        assert isinstance(loaded_yaml, dict)
        assert list(loaded_yaml) == ['local']

    def test_output_unicodebad(self):
        '''
        Tests outputter reliability with utf8
        '''
        opts = salt.config.minion_config(os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'minion'))
        opts['output_file'] = os.path.join(
            RUNTIME_VARS.TMP,
            'outputtest'
        )
        data = {'foo': {'result': False,
                        'aaa': 'azerzaeréééé',
                        'comment': u'ééééàààà'}}
        try:
            # this should not raises UnicodeEncodeError
            display_output(data, opts=opts)
        except Exception:
            # display trace in error message for debugging on jenkins
            trace = traceback.format_exc()
            sentinel = object()
            old_max_diff = getattr(self, 'maxDiff', sentinel)
            try:
                self.maxDiff = None
                self.assertEqual(trace, '')
            finally:
                if old_max_diff is sentinel:
                    delattr(self, 'maxDiff')
                else:
                    self.maxDiff = old_max_diff

    def test_output_highstate(self):
        '''
        Regression tests for the highstate outputter. Calls a basic state with various
        flags. Each comparison should be identical when successful.
        '''
        # Test basic highstate output. No frills.
        expected = ['minion:', '          ID: simple-ping', '    Function: module.run',
                    '        Name: test.ping', '      Result: True',
                    '     Comment: Module function test.ping executed',
                    '     Changes:   ', '              ret:', '                  True',
                    'Summary for minion', 'Succeeded: 1 (changed=1)', 'Failed:    0',
                    'Total states run:     1']
        state_run = self.run_salt('"minion" state.sls simple-ping')

        for expected_item in expected:
            self.assertIn(expected_item, state_run)

        # Test highstate output while also passing --out=highstate.
        # This is a regression test for Issue #29796
        state_run = self.run_salt('"minion" state.sls simple-ping --out=highstate')

        for expected_item in expected:
            self.assertIn(expected_item, state_run)

        # Test highstate output when passing --static and running a state function.
        # See Issue #44556.
        state_run = self.run_salt('"minion" state.sls simple-ping --static')

        for expected_item in expected:
            self.assertIn(expected_item, state_run)

        # Test highstate output when passing --static and --out=highstate.
        # See Issue #44556.
        state_run = self.run_salt('"minion" state.sls simple-ping --static --out=highstate')

        for expected_item in expected:
            self.assertIn(expected_item, state_run)

    def test_output_highstate_falls_back_nested(self):
        '''
        Tests outputter when passing --out=highstate with a non-state call. This should
        fall back to "nested" output.
        '''
        expected = ['minion:', '    True']
        ret = self.run_salt('"minion" test.ping --out=highstate')
        self.assertEqual(ret, expected)

    def test_static_simple(self):
        '''
        Tests passing the --static option with a basic test.ping command. This
        should be the "nested" output.
        '''
        expected = ['minion:', '    True']
        ret = self.run_salt('"minion" test.ping --static')
        self.assertEqual(ret, expected)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.integration.shell.master
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import python libs
from __future__ import absolute_import
import os
import signal
import shutil

# Import salt libs
import salt.utils.files
import salt.utils.yaml

# Import salt test libs
import tests.integration.utils
from tests.support.case import ShellCase
from tests.support.paths import TMP
from tests.support.mixins import ShellCaseCommonTestsMixin
from tests.support.unit import skipIf
from tests.integration.utils import testprogram


@skipIf(True, 'This test file should be in an isolated test space.')
class MasterTest(ShellCase, testprogram.TestProgramCase, ShellCaseCommonTestsMixin):

    _call_binary_ = 'salt-master'

    def test_issue_7754(self):
        old_cwd = os.getcwd()
        config_dir = os.path.join(TMP, 'issue-7754')
        if not os.path.isdir(config_dir):
            os.makedirs(config_dir)

        os.chdir(config_dir)

        config_file_name = 'master'
        pid_path = os.path.join(config_dir, '{0}.pid'.format(config_file_name))
        with salt.utils.files.fopen(self.get_config_file_path(config_file_name), 'r') as fhr:
            config = salt.utils.yaml.safe_load(fhr)
            config['root_dir'] = config_dir
            config['log_file'] = 'file:///tmp/log/LOG_LOCAL3'
            config['ret_port'] = config['ret_port'] + 10
            config['publish_port'] = config['publish_port'] + 10

            with salt.utils.files.fopen(os.path.join(config_dir, config_file_name), 'w') as fhw:
                salt.utils.yaml.safe_dump(config, fhw, default_flow_style=False)

        ret = self.run_script(
            self._call_binary_,
            '--config-dir {0} --pid-file {1} -l debug'.format(
                config_dir,
                pid_path
            ),
            timeout=5,
            catch_stderr=True,
            with_retcode=True
        )

        # Now kill it if still running
        if os.path.exists(pid_path):
            with salt.utils.files.fopen(pid_path) as fhr:
                try:
                    os.kill(int(fhr.read()), signal.SIGKILL)
                except OSError:
                    pass
        try:
            self.assertFalse(os.path.isdir(os.path.join(config_dir, 'file:')))
        finally:
            self.chdir(old_cwd)
            if os.path.isdir(config_dir):
                shutil.rmtree(config_dir)

    def test_exit_status_unknown_user(self):
        '''
        Ensure correct exit status when the master is configured to run as an unknown user.
        '''

        master = testprogram.TestDaemonSaltMaster(
            name='unknown_user',
            configs={'master': {'map': {'user': 'some_unknown_user_xyz'}}},
            parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        master.setup()
        stdout, stderr, status = master.run(
            args=['-d'],
            catch_stderr=True,
            with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status, 'EX_NOUSER',
                message='unknown user not on system',
                stdout=stdout,
                stderr=tests.integration.utils.decode_byte_list(stderr)
            )
        finally:
            # Although the start-up should fail, call shutdown() to set the
            # internal _shutdown flag and avoid the registered atexit calls to
            # cause timeout exeptions and respective traceback
            master.shutdown()

    # pylint: disable=invalid-name
    def test_exit_status_unknown_argument(self):
        '''
        Ensure correct exit status when an unknown argument is passed to salt-master.
        '''

        master = testprogram.TestDaemonSaltMaster(
            name='unknown_argument',
            parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        master.setup()
        stdout, stderr, status = master.run(
            args=['-d', '--unknown-argument'],
            catch_stderr=True,
            with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status, 'EX_USAGE',
                message='unknown argument',
                stdout=stdout,
                stderr=tests.integration.utils.decode_byte_list(stderr)
            )
        finally:
            # Although the start-up should fail, call shutdown() to set the
            # internal _shutdown flag and avoid the registered atexit calls to
            # cause timeout exeptions and respective traceback
            master.shutdown()

    def test_exit_status_correct_usage(self):
        '''
        Ensure correct exit status when salt-master starts correctly.
        '''

        master = testprogram.TestDaemonSaltMaster(
            name='correct_usage',
            parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        master.setup()
        stdout, stderr, status = master.run(
            args=['-d'],
            catch_stderr=True,
            with_retcode=True,
        )
        try:
            self.assert_exit_status(
                status, 'EX_OK',
                message='correct usage',
                stdout=stdout,
                stderr=tests.integration.utils.decode_byte_list(stderr)
            )
        finally:
            master.shutdown(wait_for_orphans=3)

        # Do the test again to check does master shut down correctly
        # **Due to some underlying subprocessing issues with Minion._thread_return, this
        # part of the test has been commented out. Once these underlying issues have
        # been addressed, this part of the test should be uncommented. Work for this
        # issue is being tracked in https://github.com/saltstack/salt-jenkins/issues/378
        # stdout, stderr, status = master.run(
        #     args=['-d'],
        #     catch_stderr=True,
        #     with_retcode=True,
        # )
        # try:
        #     self.assert_exit_status(
        #         status, 'EX_OK',
        #         message='correct usage',
        #         stdout=stdout,
        #         stderr=tests.integration.utils.decode_byte_list(stderr)
        #     )
        # finally:
        #     master.shutdown(wait_for_orphans=3)



# coding: utf-8

# Python libs
from __future__ import absolute_import

# Salt testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock
from tests.support.mixins import LoaderModuleMockMixin

# Salt libs
import salt.beacons.haproxy as haproxy


@skipIf(NO_MOCK, NO_MOCK_REASON)
class HAProxyBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.haproxy
    '''

    def setup_loader_modules(self):
        return {
            haproxy: {
                '__context__': {},
                '__salt__': {},
            }
        }

    def test_non_list_config(self):
        config = {}

        ret = haproxy.validate(config)

        self.assertEqual(ret, (False, 'Configuration for haproxy beacon must'
                                      ' be a list.'))

    def test_empty_config(self):
        config = [{}]

        ret = haproxy.validate(config)

        self.assertEqual(ret, (False, 'Configuration for haproxy beacon '
                                      'requires backends.'))

    def test_no_servers(self):
        config = [{'backends': {'www-backend': {'threshold': 45}}}]
        ret = haproxy.validate(config)

        self.assertEqual(ret, (False, 'Backends for haproxy '
                                      'beacon require servers.'))

    def test_threshold_reached(self):
        config = [{'backends': {'www-backend': {'threshold': 45,
                                'servers': ['web1']
                                   }}}]
        ret = haproxy.validate(config)

        self.assertEqual(ret, (True, 'Valid beacon configuration'))

        mock = MagicMock(return_value=46)
        with patch.dict(haproxy.__salt__, {'haproxy.get_sessions': mock}):
            ret = haproxy.beacon(config)
            self.assertEqual(ret, [{'threshold': 45,
                                    'scur': 46,
                                    'server': 'web1'}])

    def test_threshold_not_reached(self):
        config = [{'backends': {'www-backend': {'threshold': 100,
                                'servers': ['web1']
                                   }}}]
        ret = haproxy.validate(config)

        self.assertEqual(ret, (True, 'Valid beacon configuration'))

        mock = MagicMock(return_value=50)
        with patch.dict(haproxy.__salt__, {'haproxy.get_sessions': mock}):
            ret = haproxy.beacon(config)
            self.assertEqual(ret, [])



# -*- coding: utf-8 -*-
'''
    tests.unit.version_test
    ~~~~~~~~~~~~~~~~~~~~~~~

    These tests are copied from python's source `Lib/distutils/tests/test_version.py`
    Some new examples were added and some adjustments were made to run tests in python 2 and 3
'''
# pylint: disable=string-substitution-usage-error

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import sys
import warnings

# Import Salt Testing libs
import tests.integration as integration
from tests.support.unit import TestCase, skipIf
from tests.support.mock import patch, NO_MOCK, NO_MOCK_REASON

# Import Salt libs
import salt.modules.cmdmod
import salt.version
import salt.utils.platform
import salt.utils.versions
from salt.utils.versions import LooseVersion, StrictVersion

# Import 3rd-party libs
from salt.ext import six

if six.PY2:
    cmp_method = '__cmp__'
else:
    cmp_method = '_cmp'


class VersionTestCase(TestCase):

    def test_prerelease(self):
        version = StrictVersion('1.2.3a1')
        self.assertEqual(version.version, (1, 2, 3))
        self.assertEqual(version.prerelease, ('a', 1))
        self.assertEqual(six.text_type(version), '1.2.3a1')

        version = StrictVersion('1.2.0')
        self.assertEqual(six.text_type(version), '1.2')

    def test_cmp_strict(self):
        versions = (('1.5.1', '1.5.2b2', -1),
                    ('161', '3.10a', ValueError),
                    ('8.02', '8.02', 0),
                    ('3.4j', '1996.07.12', ValueError),
                    ('3.2.pl0', '3.1.1.6', ValueError),
                    ('2g6', '11g', ValueError),
                    ('0.9', '2.2', -1),
                    ('1.2.1', '1.2', 1),
                    ('1.1', '1.2.2', -1),
                    ('1.2', '1.1', 1),
                    ('1.2.1', '1.2.2', -1),
                    ('1.2.2', '1.2', 1),
                    ('1.2', '1.2.2', -1),
                    ('0.4.0', '0.4', 0),
                    ('1.13++', '5.5.kw', ValueError),
                    # Added by us
                    ('1.1.1a1', '1.1.1', -1)
                    )

        for v1, v2, wanted in versions:
            try:
                res = getattr(StrictVersion(v1), cmp_method)(StrictVersion(v2))
            except ValueError:
                if wanted is ValueError:
                    continue
                else:
                    raise AssertionError(("cmp(%s, %s) "
                                          "shouldn't raise ValueError") % (v1, v2))
            self.assertEqual(res, wanted,
                             'cmp(%s, %s) should be %s, got %s' %
                             (v1, v2, wanted, res))

    def test_cmp(self):
        versions = (('1.5.1', '1.5.2b2', -1),
                    ('161', '3.10a', 1),
                    ('8.02', '8.02', 0),
                    ('3.4j', '1996.07.12', -1),
                    ('3.2.pl0', '3.1.1.6', 1),
                    ('2g6', '11g', -1),
                    ('0.960923', '2.2beta29', -1),
                    ('1.13++', '5.5.kw', -1),
                    # Added by us
                    ('3.10.0-514.el7', '3.10.0-514.6.1.el7', 1),
                    ('2.2.2', '2.12.1', -1)
                    )

        for v1, v2, wanted in versions:
            res = getattr(LooseVersion(v1), cmp_method)(LooseVersion(v2))
            self.assertEqual(res, wanted,
                             'cmp(%s, %s) should be %s, got %s' %
                             (v1, v2, wanted, res))

    @skipIf(not salt.utils.platform.is_linux(), 'only need to run on linux')
    def test_spelling_version_name(self):
        '''
        check the spelling of the version name for the release
        names in the salt.utils.versions.warn_until call
        '''
        salt_dir = integration.CODE_DIR
        query = 'salt.utils.versions.warn_until'
        names = salt.version.SaltStackVersion.NAMES

        salt_dir += '/salt/'
        cmd = 'grep -lr {0} -A 1 '.format(query) + salt_dir

        grep_call = salt.modules.cmdmod.run_stdout(cmd=cmd).split(os.linesep)

        for line in grep_call:
            num_cmd = salt.modules.cmdmod.run_stdout('grep -c {0} {1}'.format(query, line))
            ver_cmd = salt.modules.cmdmod.run_stdout('grep {0} {1} -A 1'.format(query, line))
            if 'pyc' in line:
                break

            match = 0
            for key in names:
                if key in ver_cmd:
                    match = match + (ver_cmd.count(key))
            if 'utils/__init__.py' in line:
                # work around for utils/__init__.py because
                # it includes the warn_utils function
                match = match + 1
            self.assertEqual(match, int(num_cmd), msg='The file: {0} has an '
                             'incorrect spelling for the release name in the warn_utils '
                             'call: {1}. Expecting one of these release names: '
                             '{2}'.format(line, ver_cmd, names))


class VersionFuncsTestCase(TestCase):

    @skipIf(NO_MOCK, NO_MOCK_REASON)
    def test_compare(self):
        ret = salt.utils.versions.compare('1.0', '==', '1.0')
        self.assertTrue(ret)

        ret = salt.utils.versions.compare('1.0', '!=', '1.0')
        self.assertFalse(ret)

        with patch.object(salt.utils.versions, 'log') as log_mock:
            ret = salt.utils.versions.compare('1.0', 'HAH I AM NOT A COMP OPERATOR! I AM YOUR FATHER!', '1.0')
            self.assertTrue(log_mock.error.called)

    def test_kwargs_warn_until(self):
        # Test invalid version arg
        self.assertRaises(RuntimeError, salt.utils.versions.kwargs_warn_until, {}, [])

    def test_warn_until_warning_raised(self):
        # We *always* want *all* warnings thrown on this module
        warnings.filterwarnings('always', '', DeprecationWarning, __name__)

        def raise_warning(_version_info_=(0, 16, 0)):
            salt.utils.versions.warn_until(
                (0, 17), 'Deprecation Message!',
                _version_info_=_version_info_

            )

        def raise_named_version_warning(_version_info_=(0, 16, 0)):
            salt.utils.versions.warn_until(
                'Hydrogen', 'Deprecation Message!',
                _version_info_=_version_info_
            )

        # raise_warning should show warning until version info is >= (0, 17)
        with warnings.catch_warnings(record=True) as recorded_warnings:
            raise_warning()
            self.assertEqual(
                'Deprecation Message!', six.text_type(recorded_warnings[0].message)
            )

        # raise_warning should show warning until version info is >= (0, 17)
        with warnings.catch_warnings(record=True) as recorded_warnings:
            raise_named_version_warning()
            self.assertEqual(
                'Deprecation Message!', six.text_type(recorded_warnings[0].message)
            )

        # the deprecation warning is not issued because we passed
        # _dont_call_warning
        with warnings.catch_warnings(record=True) as recorded_warnings:
            salt.utils.versions.warn_until(
                (0, 17), 'Foo', _dont_call_warnings=True,
                _version_info_=(0, 16)
            )
            self.assertEqual(0, len(recorded_warnings))

        # Let's set version info to (0, 17), a RuntimeError should be raised
        with self.assertRaisesRegex(
                RuntimeError,
                r'The warning triggered on filename \'(.*)test_versions.py\', '
                r'line number ([\d]+), is supposed to be shown until version '
                r'0.17.0 is released. Current version is now 0.17.0. '
                r'Please remove the warning.'):
            raise_warning(_version_info_=(0, 17, 0))

        # Let's set version info to (0, 17), a RuntimeError should be raised
        with self.assertRaisesRegex(
                RuntimeError,
                r'The warning triggered on filename \'(.*)test_versions.py\', '
                r'line number ([\d]+), is supposed to be shown until version '
                r'(.*) is released. Current version is now '
                r'([\d.]+). Please remove the warning.'):
            raise_named_version_warning(_version_info_=(getattr(sys, 'maxint', None) or getattr(sys, 'maxsize'), 16, 0))

        # Even though we're calling warn_until, we pass _dont_call_warnings
        # because we're only after the RuntimeError
        with self.assertRaisesRegex(
                RuntimeError,
                r'The warning triggered on filename \'(.*)test_versions.py\', '
                r'line number ([\d]+), is supposed to be shown until version '
                r'0.17.0 is released. Current version is now '
                r'(.*). Please remove the warning.'):
            salt.utils.versions.warn_until(
                (0, 17), 'Foo', _dont_call_warnings=True
            )

        with self.assertRaisesRegex(
                RuntimeError,
                r'The warning triggered on filename \'(.*)test_versions.py\', '
                r'line number ([\d]+), is supposed to be shown until version '
                r'(.*) is released. Current version is now '
                r'(.*). Please remove the warning.'):
            salt.utils.versions.warn_until(
                'Hydrogen', 'Foo', _dont_call_warnings=True,
                _version_info_=(getattr(sys, 'maxint', None) or getattr(sys, 'maxsize'), 16, 0)
            )

        # version on the deprecation message gets properly formatted
        with warnings.catch_warnings(record=True) as recorded_warnings:
            vrs = salt.version.SaltStackVersion.from_name('Helium')
            salt.utils.versions.warn_until(
                'Helium', 'Deprecation Message until {version}!',
                _version_info_=(vrs.major - 1, 0)
            )
            self.assertEqual(
                'Deprecation Message until {0}!'.format(vrs.formatted_version),
                six.text_type(recorded_warnings[0].message)
            )

    def test_kwargs_warn_until_warning_raised(self):
        # We *always* want *all* warnings thrown on this module
        warnings.filterwarnings('always', '', DeprecationWarning, __name__)

        def raise_warning(**kwargs):
            _version_info_ = kwargs.pop('_version_info_', (0, 16, 0))
            salt.utils.versions.kwargs_warn_until(
                kwargs,
                (0, 17),
                _version_info_=_version_info_
            )

        # raise_warning({...}) should show warning until version info is >= (0, 17)
        with warnings.catch_warnings(record=True) as recorded_warnings:
            raise_warning(foo=42)  # with a kwarg
            self.assertEqual(
                'The following parameter(s) have been deprecated and '
                'will be removed in \'0.17.0\': \'foo\'.',
                six.text_type(recorded_warnings[0].message)
            )
        # With no **kwargs, should not show warning until version info is >= (0, 17)
        with warnings.catch_warnings(record=True) as recorded_warnings:
            salt.utils.versions.kwargs_warn_until(
                {},  # no kwargs
                (0, 17),
                _version_info_=(0, 16, 0)
            )
            self.assertEqual(0, len(recorded_warnings))

        # Let's set version info to (0, 17), a RuntimeError should be raised
        # regardless of whether or not we pass any **kwargs.
        with self.assertRaisesRegex(
                RuntimeError,
                r'The warning triggered on filename \'(.*)test_versions.py\', '
                r'line number ([\d]+), is supposed to be shown until version '
                r'0.17.0 is released. Current version is now 0.17.0. '
                r'Please remove the warning.'):
            raise_warning(_version_info_=(0, 17))  # no kwargs

        with self.assertRaisesRegex(
                RuntimeError,
                r'The warning triggered on filename \'(.*)test_versions.py\', '
                r'line number ([\d]+), is supposed to be shown until version '
                r'0.17.0 is released. Current version is now 0.17.0. '
                r'Please remove the warning.'):
            raise_warning(bar='baz', qux='quux', _version_info_=(0, 17))  # some kwargs



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Libs
import salt.modules.win_powercfg as powercfg

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch,
    call
)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PowerCfgTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Validate the powercfg state
    '''

    def setup_loader_modules(self):
        return {powercfg: {'__grains__': {'osrelease': 8}}}

    query_output = '''Subgroup GUID: 238c9fa8-0aad-41ed-83f4-97be242c8f20  (Hibernate)
                GUID Alias: SUB_SLEEP
                Power Setting GUID: 29f6c1db-86da-48c5-9fdb-f2b67b1f44da  (Hibernate after)
                GUID Alias: HIBERNATEIDLE
                Minimum Possible Setting: 0x00000000
                Maximum Possible Setting: 0xffffffff
                Possible Settings increment: 0x00000001
                Possible Settings units: Seconds
                Current AC Power Setting Index: 0x00000708
                Current DC Power Setting Index: 0x00000384'''

    def test_set_monitor_timeout(self):
        '''
        Test to make sure we can set the monitor timeout value
        '''
        mock = MagicMock(return_value=0)
        mock.side_effect = [
            'Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)',
            self.query_output]

        mock_retcode = MagicMock(return_value=0)

        with patch.dict(powercfg.__salt__, {'cmd.run': mock}):
            with patch.dict(powercfg.__salt__, {'cmd.retcode': mock_retcode}):
                powercfg.set_monitor_timeout(0, 'dc')
                mock.assert_called_once_with(
                    'powercfg /getactivescheme',
                    python_shell=False)
                mock_retcode.assert_called_once_with(
                    'powercfg /setdcvalueindex 381b4222-f694-41f0-9685-ff5bb260df2e SUB_VIDEO VIDEOIDLE 0',
                    python_shell=False)

    def test_set_disk_timeout(self):
        '''
        Test to make sure we can set the disk timeout value
        '''
        mock = MagicMock()
        mock.side_effect = [
            'Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)',
            self.query_output]

        mock_retcode = MagicMock(return_value=0)

        with patch.dict(powercfg.__salt__, {'cmd.run': mock}):
            with patch.dict(powercfg.__salt__, {'cmd.retcode': mock_retcode}):
                powercfg.set_disk_timeout(0, 'dc')
                mock.assert_called_once_with(
                    'powercfg /getactivescheme',
                    python_shell=False)
                mock_retcode.assert_called_once_with(
                    'powercfg /setdcvalueindex 381b4222-f694-41f0-9685-ff5bb260df2e SUB_DISK DISKIDLE 0',
                    python_shell=False)

    def test_set_standby_timeout(self):
        '''
        Test to make sure we can set the standby timeout value
        '''
        mock = MagicMock(return_value=0)
        mock.side_effect = [
            'Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)',
            self.query_output]

        mock_retcode = MagicMock(return_value=0)

        with patch.dict(powercfg.__salt__, {'cmd.run': mock}):
            with patch.dict(powercfg.__salt__, {'cmd.retcode': mock_retcode}):
                powercfg.set_standby_timeout(0, 'dc')
                mock.assert_called_once_with(
                    'powercfg /getactivescheme',
                    python_shell=False)
                mock_retcode.assert_called_once_with(
                    'powercfg /setdcvalueindex 381b4222-f694-41f0-9685-ff5bb260df2e SUB_SLEEP STANDBYIDLE 0',
                    python_shell=False)

    def test_set_hibernate_timeout(self):
        '''
        Test to make sure we can set the hibernate timeout value
        '''
        mock = MagicMock(return_value=0)
        mock.side_effect = [
            'Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)',
            self.query_output]

        mock_retcode = MagicMock(return_value=0)

        with patch.dict(powercfg.__salt__, {'cmd.run': mock}):
            with patch.dict(powercfg.__salt__, {'cmd.retcode': mock_retcode}):
                powercfg.set_hibernate_timeout(0, 'dc')
                mock.assert_called_once_with(
                    'powercfg /getactivescheme',
                    python_shell=False)
                mock_retcode.assert_called_once_with(
                    'powercfg /setdcvalueindex 381b4222-f694-41f0-9685-ff5bb260df2e SUB_SLEEP HIBERNATEIDLE 0',
                    python_shell=False)

    def test_get_monitor_timeout(self):
        '''
        Test to make sure we can get the monitor timeout value
        '''
        mock = MagicMock()
        mock.side_effect = [
            'Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)',
            self.query_output]

        with patch.dict(powercfg.__salt__, {'cmd.run': mock}):
            ret = powercfg.get_monitor_timeout()
            calls = [
                call('powercfg /getactivescheme', python_shell=False),
                call('powercfg /q 381b4222-f694-41f0-9685-ff5bb260df2e SUB_VIDEO VIDEOIDLE',
                     python_shell=False)
            ]
            mock.assert_has_calls(calls)

            self.assertEqual({'ac': 30, 'dc': 15}, ret)

    def test_get_disk_timeout(self):
        '''
        Test to make sure we can get the disk timeout value
        '''
        mock = MagicMock()
        mock.side_effect = [
            'Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)',
            self.query_output]

        with patch.dict(powercfg.__salt__, {'cmd.run': mock}):
            ret = powercfg.get_disk_timeout()
            calls = [
                call('powercfg /getactivescheme', python_shell=False),
                call('powercfg /q 381b4222-f694-41f0-9685-ff5bb260df2e SUB_DISK DISKIDLE',
                     python_shell=False)
            ]
            mock.assert_has_calls(calls)

            self.assertEqual({'ac': 30, 'dc': 15}, ret)

    def test_get_standby_timeout(self):
        '''
        Test to make sure we can get the standby timeout value
        '''
        mock = MagicMock()
        mock.side_effect = [
            'Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)',
            self.query_output]

        with patch.dict(powercfg.__salt__, {'cmd.run': mock}):
            ret = powercfg.get_standby_timeout()
            calls = [
                call('powercfg /getactivescheme', python_shell=False),
                call('powercfg /q 381b4222-f694-41f0-9685-ff5bb260df2e SUB_SLEEP STANDBYIDLE',
                     python_shell=False)
            ]
            mock.assert_has_calls(calls)

            self.assertEqual({'ac': 30, 'dc': 15}, ret)

    def test_get_hibernate_timeout(self):
        '''
        Test to make sure we can get the hibernate timeout value
        '''
        mock = MagicMock()
        mock.side_effect = [
            'Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)',
            self.query_output]

        with patch.dict(powercfg.__salt__, {'cmd.run': mock}):
            ret = powercfg.get_hibernate_timeout()
            calls = [
                call('powercfg /getactivescheme', python_shell=False),
                call('powercfg /q 381b4222-f694-41f0-9685-ff5bb260df2e SUB_SLEEP HIBERNATEIDLE',
                     python_shell=False)
            ]
            mock.assert_has_calls(calls)

            self.assertEqual({'ac': 30, 'dc': 15}, ret)

    def test_windows_7(self):
        '''
        Test to make sure we can get the hibernate timeout value on windows 7
        '''
        mock = MagicMock()
        mock.side_effect = [
            'Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)',
            self.query_output]

        with patch.dict(powercfg.__salt__, {'cmd.run': mock}):
            with patch.dict(powercfg.__grains__, {'osrelease': '7'}):
                ret = powercfg.get_hibernate_timeout()
                calls = [
                    call('powercfg /getactivescheme', python_shell=False),
                    call('powercfg /q 381b4222-f694-41f0-9685-ff5bb260df2e SUB_SLEEP',
                         python_shell=False)
                ]
                mock.assert_has_calls(calls)

                self.assertEqual({'ac': 30, 'dc': 15}, ret)

    def test_set_hibernate_timeout_scheme(self):
        '''
        Test to make sure we can set the hibernate timeout value
        '''
        mock = MagicMock(return_value=0)
        mock.side_effect = [self.query_output]

        with patch.dict(powercfg.__salt__, {'cmd.retcode': mock}):
            powercfg.set_hibernate_timeout(0, 'dc', scheme='SCHEME_MIN')
            mock.assert_called_once_with(
                'powercfg /setdcvalueindex SCHEME_MIN SUB_SLEEP HIBERNATEIDLE 0',
                python_shell=False)

    def test_get_hibernate_timeout_scheme(self):
        '''
        Test to make sure we can get the hibernate timeout value with a
        specified scheme
        '''
        mock = MagicMock()
        mock.side_effect = [self.query_output]

        with patch.dict(powercfg.__salt__, {'cmd.run': mock}):
            ret = powercfg.get_hibernate_timeout(scheme='SCHEME_MIN')
            mock.assert_called_once_with(
                'powercfg /q SCHEME_MIN SUB_SLEEP HIBERNATEIDLE',
                python_shell=False)

            self.assertEqual({'ac': 30, 'dc': 15}, ret)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
    :copyright: Copyright 2017 by the SaltStack Team, see AUTHORS for more details.


    tests.unit.beacons.test_status
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Status beacon test cases
'''

# Python libs
from __future__ import absolute_import
import sys

# Salt libs
import salt.config
import salt.loader
from salt.beacons import status
import salt.modules.status as status_module

# Salt testing libs
from tests.support.unit import TestCase
from tests.support.mixins import LoaderModuleMockMixin


class StatusBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.status
    '''

    def setup_loader_modules(self):
        opts = salt.config.DEFAULT_MINION_OPTS
        module_globals = {
            '__opts__': opts,
            '__salt__': 'autoload',
            '__context__': {},
            '__grains__': {'kernel': 'Linux'}
        }
        return {
            status: module_globals,
            status_module: module_globals
        }

    def test_empty_config(self, *args, **kwargs):
        config = []

        ret = status.validate(config)
        self.assertEqual(ret, (True, 'Valid beacon configuration'))

        ret = status.beacon(config)

        if sys.platform.startswith('win'):
            expected = []
        else:
            expected = sorted(['loadavg', 'meminfo', 'cpustats', 'vmstats', 'time'])

        self.assertEqual(sorted(list(ret[0]['data'])), expected)

    def test_deprecated_dict_config(self):
        config = {'time': ['all']}

        ret = status.validate(config)
        self.assertEqual(ret, (False, 'Configuration for status beacon must be a list.'))

    def test_list_config(self):
        config = [{'time': ['all']}]

        ret = status.validate(config)
        self.assertEqual(ret, (True, 'Valid beacon configuration'))

        ret = status.beacon(config)

        if sys.platform.startswith('win'):
            expected = []
        else:
            expected = ['time']

        self.assertEqual(list(ret[0]['data']), expected)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function
import datetime
import logging
import os
import signal
import subprocess
import textwrap

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf
from tests.support.helpers import destructiveTest, skip_if_not_root, flaky

# Import Salt libs
import salt.utils.files
import salt.utils.path
import salt.utils.platform
import salt.states.file
from salt.ext.six.moves import range
from salt.ext import six

log = logging.getLogger(__name__)


@skipIf(not salt.utils.platform.is_linux(), 'These tests can only be run on linux')
class SystemModuleTest(ModuleCase):
    '''
    Validate the date/time functions in the system module
    '''
    fmt_str = "%Y-%m-%d %H:%M:%S"

    def __init__(self, arg):
        super(self.__class__, self).__init__(arg)
        self._orig_time = None
        self._machine_info = True

    def setUp(self):
        super(SystemModuleTest, self).setUp()
        os_grain = self.run_function('grains.item', ['kernel'])
        if os_grain['kernel'] not in 'Linux':
            self.skipTest(
                'Test not applicable to \'{kernel}\' kernel'.format(
                    **os_grain
                )
            )
        if self.run_function('service.available', ['systemd-timesyncd']):
            self.run_function('service.stop', ['systemd-timesyncd'])

    def tearDown(self):
        if self._orig_time is not None:
            self._restore_time()
        self._orig_time = None
        if self._machine_info is not True:
            self._restore_machine_info()
        self._machine_info = True
        if self.run_function('service.available', ['systemd-timesyncd']):
            self.run_function('service.start', ['systemd-timesyncd'])

    def _save_time(self):
        self._orig_time = datetime.datetime.utcnow()

    def _set_time(self, new_time, offset=None):
        t = new_time.timetuple()[:6]
        t += (offset,)
        return self.run_function('system.set_system_date_time', t)

    def _restore_time(self):
        result = self._set_time(self._orig_time, "+0000")
        self.assertTrue(result, msg="Unable to restore time properly")

    def _same_times(self, t1, t2, seconds_diff=30):
        '''
        Helper function to check if two datetime objects
        are close enough to the same time.
        '''
        return abs(t1 - t2) < datetime.timedelta(seconds=seconds_diff)

    def _hwclock_has_compare(self):
        '''
        Some builds of hwclock don't include the `--compare` function
        needed to test hw/sw clock synchronization. Returns false on
        systems where it's not present so that we can skip the
        comparison portion of the test.
        '''
        res = self.run_function('cmd.run_all', cmd='hwclock -h')
        return res['retcode'] == 0 and res['stdout'].find('--compare') > 0

    def _test_hwclock_sync(self):
        '''
        Check that hw and sw clocks are sync'd.
        '''
        if not self.run_function('system.has_settable_hwclock'):
            return None
        if not self._hwclock_has_compare():
            return None

        class CompareTimeout(BaseException):
            pass

        def _alrm_handler(sig, frame):
            log.warning('hwclock --compare failed to produce output after 3 seconds')
            raise CompareTimeout

        for _ in range(2):
            try:
                orig_handler = signal.signal(signal.SIGALRM, _alrm_handler)
                signal.alarm(3)
                rpipeFd, wpipeFd = os.pipe()
                log.debug('Comparing hwclock to sys clock')
                with os.fdopen(rpipeFd, "r") as rpipe:
                    with os.fdopen(wpipeFd, "w") as wpipe:
                        with salt.utils.files.fopen(os.devnull, "r") as nulFd:
                            p = subprocess.Popen(args=['hwclock', '--compare'],
                                stdin=nulFd, stdout=wpipeFd, stderr=subprocess.PIPE)
                            p.communicate()

                            # read header
                            rpipe.readline()

                            # read first time comparison
                            timeCompStr = rpipe.readline()

                            # stop
                            p.terminate()

                            timeComp = timeCompStr.split()
                            hwTime = float(timeComp[0])
                            swTime = float(timeComp[1])
                            diff = abs(hwTime - swTime)

                            self.assertTrue(diff <= 2.0,
                                msg=("hwclock difference too big: " + six.text_type(timeCompStr)))
                            break
            except CompareTimeout:
                p.terminate()
            finally:
                signal.alarm(0)
                signal.signal(signal.SIGALRM, orig_handler)
        else:
            log.error('Failed to check hwclock sync')

    def _save_machine_info(self):
        if os.path.isfile('/etc/machine-info'):
            with salt.utils.files.fopen('/etc/machine-info', 'r') as mach_info:
                self._machine_info = mach_info.read()
        else:
            self._machine_info = False

    def _restore_machine_info(self):
        if self._machine_info is not False:
            with salt.utils.files.fopen('/etc/machine-info', 'w') as mach_info:
                mach_info.write(self._machine_info)
        else:
            self.run_function('file.remove', ['/etc/machine-info'])

    def test_get_system_date_time(self):
        '''
        Test we are able to get the correct time
        '''
        t1 = datetime.datetime.now()
        res = self.run_function('system.get_system_date_time')
        t2 = datetime.datetime.strptime(res, self.fmt_str)
        msg = ("Difference in times is too large. Now: {0} Fake: {1}"
               .format(t1, t2))
        self.assertTrue(self._same_times(t1, t2, seconds_diff=2), msg=msg)

    def test_get_system_date_time_utc(self):
        '''
        Test we are able to get the correct time with utc
        '''
        t1 = datetime.datetime.utcnow()
        res = self.run_function('system.get_system_date_time',
                                utc_offset="+0000")
        t2 = datetime.datetime.strptime(res, self.fmt_str)
        msg = ("Difference in times is too large. Now: {0} Fake: {1}"
               .format(t1, t2))
        self.assertTrue(self._same_times(t1, t2, seconds_diff=2), msg=msg)

    @destructiveTest
    @skip_if_not_root
    def test_set_system_date_time(self):
        '''
        Test changing the system clock. We are only able to set it up to a
        resolution of a second so this test may appear to run in negative time.
        '''
        self._save_time()
        cmp_time = datetime.datetime.now() - datetime.timedelta(days=7)
        result = self._set_time(cmp_time)
        time_now = datetime.datetime.now()

        msg = ("Difference in times is too large. Now: {0} Fake: {1}"
               .format(time_now, cmp_time))
        self.assertTrue(result and self._same_times(time_now, cmp_time),
                        msg=msg)
        self._test_hwclock_sync()

    @destructiveTest
    @skip_if_not_root
    def test_set_system_date_time_utc(self):
        '''
        Test changing the system clock. We are only able to set it up to a
        resolution of a second so this test may appear to run in negative time.
        '''
        self._save_time()
        cmp_time = datetime.datetime.utcnow() - datetime.timedelta(days=7)
        result = self._set_time(cmp_time, offset="+0000")
        time_now = datetime.datetime.utcnow()

        msg = ("Difference in times is too large. Now: {0} Fake: {1}"
               .format(time_now, cmp_time))
        self.assertTrue(result)
        self.assertTrue(self._same_times(time_now, cmp_time), msg=msg)
        self._test_hwclock_sync()

    @destructiveTest
    @skip_if_not_root
    def test_set_system_date_time_utcoffset_east(self):
        '''
        Test changing the system clock. We are only able to set it up to a
        resolution of a second so this test may appear to run in negative time.
        '''
        self._save_time()
        cmp_time = datetime.datetime.utcnow() - datetime.timedelta(days=7)
        # 25200 seconds = 7 hours
        time_to_set = cmp_time - datetime.timedelta(seconds=25200)
        result = self._set_time(time_to_set, offset='-0700')
        time_now = datetime.datetime.utcnow()

        msg = ("Difference in times is too large. Now: {0} Fake: {1}"
               .format(time_now, cmp_time))
        self.assertTrue(result)
        self.assertTrue(self._same_times(time_now, cmp_time), msg=msg)
        self._test_hwclock_sync()

    @destructiveTest
    @skip_if_not_root
    def test_set_system_date_time_utcoffset_west(self):
        '''
        Test changing the system clock. We are only able to set it up to a
        resolution of a second so this test may appear to run in negative time.
        '''
        self._save_time()
        cmp_time = datetime.datetime.utcnow() - datetime.timedelta(days=7)
        # 7200 seconds = 2 hours
        time_to_set = cmp_time + datetime.timedelta(seconds=7200)
        result = self._set_time(time_to_set, offset='+0200')
        time_now = datetime.datetime.utcnow()

        msg = ("Difference in times is too large. Now: {0} Fake: {1}"
               .format(time_now, cmp_time))
        self.assertTrue(result)
        self.assertTrue(self._same_times(time_now, cmp_time), msg=msg)
        self._test_hwclock_sync()

    @flaky
    @destructiveTest
    @skip_if_not_root
    def test_set_system_time(self):
        '''
        Test setting the system time without adjusting the date.
        '''
        cmp_time = datetime.datetime.now().replace(hour=10, minute=5, second=0)
        self._save_time()

        result = self.run_function('system.set_system_time', ["10:05:00"])

        time_now = datetime.datetime.now()
        msg = ("Difference in times is too large. Now: {0} Fake: {1}"
               .format(time_now, cmp_time))

        self.assertTrue(result)
        self.assertTrue(self._same_times(time_now, cmp_time), msg=msg)
        self._test_hwclock_sync()

    @destructiveTest
    @skip_if_not_root
    def test_set_system_date(self):
        '''
        Test setting the system date without adjusting the time.
        '''
        cmp_time = datetime.datetime.now() - datetime.timedelta(days=7)

        self._save_time()
        result = self.run_function(
            'system.set_system_date',
            [cmp_time.strftime('%Y-%m-%d')]
        )

        time_now = datetime.datetime.now()
        msg = ("Difference in times is too large. Now: {0} Fake: {1}"
               .format(time_now, cmp_time))

        self.assertTrue(result)
        self.assertTrue(self._same_times(time_now, cmp_time), msg=msg)
        self._test_hwclock_sync()

    @skip_if_not_root
    def test_get_computer_desc(self):
        '''
        Test getting the system hostname
        '''
        res = self.run_function('system.get_computer_desc')

        hostname_cmd = salt.utils.path.which('hostnamectl')
        if hostname_cmd:
            desc = self.run_function('cmd.run', ["hostnamectl status --pretty"])
            self.assertEqual(res, desc)
        else:
            if not os.path.isfile('/etc/machine-info'):
                self.assertFalse(res)
            else:
                with salt.utils.files.fopen('/etc/machine-info', 'r') as mach_info:
                    data = mach_info.read()
                    self.assertIn(res, data.decode('string_escape'))

    @destructiveTest
    @skip_if_not_root
    def test_set_computer_desc(self):
        '''
        Test setting the computer description
        '''
        self._save_machine_info()
        desc = "test"
        ret = self.run_function('system.set_computer_desc', [desc])
        computer_desc = self.run_function('system.get_computer_desc')

        self.assertTrue(ret)
        self.assertIn(desc, computer_desc)

    @destructiveTest
    @skip_if_not_root
    def test_set_computer_desc_multiline(self):
        '''
        Test setting the computer description with a multiline string with tabs
        and double-quotes.
        '''
        self._save_machine_info()
        desc = textwrap.dedent('''\
            'First Line
            \tSecond Line: 'single-quoted string'
            \t\tThird Line: "double-quoted string with unicode: питон"''')
        ret = self.run_function('system.set_computer_desc', [desc])
        # self.run_function returns the serialized return, we need to convert
        # back to unicode to compare to desc. in the assertIn below.
        computer_desc = salt.utils.stringutils.to_unicode(
            self.run_function('system.get_computer_desc')
        )

        self.assertTrue(ret)
        self.assertIn(desc, computer_desc)

    @skip_if_not_root
    def test_has_hwclock(self):
        '''
        Verify platform has a settable hardware clock, if possible.
        '''
        if self.run_function('grains.get', ['os_family']) == 'NILinuxRT':
            self.assertTrue(self.run_function('system._has_settable_hwclock'))
            self.assertTrue(self._hwclock_has_compare())


@skipIf(not salt.utils.platform.is_windows(), 'These tests can only be run on windows')
class WinSystemModuleTest(ModuleCase):
    '''
    Validate the date/time functions in the win_system module
    '''

    @classmethod
    def tearDownClass(cls):
        if subprocess.call('w32tm /resync', shell=True) != 0:
            log.error("Re-syncing time failed")

    def test_get_computer_name(self):
        '''
        Test getting the computer name
        '''
        ret = self.run_function('system.get_computer_name')

        self.assertTrue(isinstance(ret, six.text_type))
        import socket
        name = socket.gethostname()
        self.assertEqual(name, ret)

    @destructiveTest
    def test_set_computer_desc(self):
        '''
        Test setting the computer description
        '''
        desc = 'test description'
        set_desc = self.run_function('system.set_computer_desc', [desc])
        self.assertTrue(set_desc)

        get_desc = self.run_function('system.get_computer_desc')
        self.assertEqual(set_desc['Computer Description'], get_desc)

    def test_get_system_time(self):
        '''
        Test getting the system time
        '''
        ret = self.run_function('system.get_system_time')
        now = datetime.datetime.now()
        self.assertEqual(now.strftime("%I:%M"), ret.rsplit(':', 1)[0])

    @flaky
    @destructiveTest
    @flaky
    def test_set_system_time(self):
        '''
        Test setting the system time
        '''
        test_time = '10:55'
        set_time = self.run_function('system.set_system_time', [test_time + ' AM'])
        get_time = self.run_function('system.get_system_time').rsplit(':', 1)[0]
        self.assertEqual(get_time, test_time)

    def test_get_system_date(self):
        '''
        Test getting system date
        '''
        ret = self.run_function('system.get_system_date')
        date = datetime.datetime.now().date().strftime("%m/%d/%Y")
        self.assertEqual(date, ret)

    @destructiveTest
    def test_set_system_date(self):
        '''
        Test setting system date
        '''
        self.assertTrue(self.run_function('system.set_system_date', ['3/25/2018']))
